@@ -43,7 +43,6 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <unistd.h>
-#include <assert.h>
 #include <string.h>
 #include <fcntl.h>
 #include <errno.h>
@@ -57,6 +56,7 @@
 #include "rpc.h"
 #include "bulk.h"
 #include "time.h"
+#include "exitcode.h"
 
 #include "timeval.h"
 
@@ -95,14 +95,14 @@ int dcc_open_read(const char *fname, int *ifd, off_t *fsize)
             return 0;
         } else {
             rs_log_error("failed to open %s: %s", fname, strerror(save_errno));
-            return -1;
+            return EXIT_IO_ERROR;
         }
     }
 
     if (fstat(*ifd, &buf) == -1) {
 	rs_log_error("fstat %s failed: %s", fname, strerror(errno));
-        close(*ifd);
-        return -1;
+        dcc_close(*ifd);
+        return EXIT_IO_ERROR;
     }
 
     *fsize = buf.st_size;
@@ -118,6 +118,9 @@ static void dcc_calc_rate(size_t size_out,
                           double *rate)
 {
     struct timeval delta;
+
+    /* FIXME: Protect against division by zero and other floating-point
+     * exceptions. */
     
     timeval_subtract(&delta, after, before);
 
@@ -152,7 +155,7 @@ int dcc_x_file_timed(int ofd, const char *fname, const char *token,
         double secs, rate;
         
         dcc_calc_rate(size, &before, &after, &secs, &rate);
-        rs_log_info("%ld bytes sent in %.3fs, rate %.0fkB/s",
+        rs_log_info("%ld bytes sent in %.6fs, rate %.0fkB/s",
                     (long) size, secs, rate);
     }
 
@@ -177,7 +180,7 @@ int dcc_x_file(int ofd, const char *fname, const char *token,
     off_t f_size; 
 
     if (dcc_open_read(fname, &ifd, &f_size))
-        return -1;
+        return EXIT_IO_ERROR;
 
     rs_trace("send %ld byte file %s with token %s",
              (long) f_size, fname, token);
@@ -199,13 +202,13 @@ int dcc_x_file(int ofd, const char *fname, const char *token,
     }
 
     if (ifd != -1)
-        close(ifd);
+        dcc_close(ifd);
     return 0;
 
   failed:
     if (ifd != -1)
-        close(ifd);
-    return -1;
+        dcc_close(ifd);
+    return EXIT_IO_ERROR;
 }
 
 
@@ -224,13 +227,13 @@ int dcc_r_file(int ifd, const char *filename, size_t len)
 
     if (unlink(filename) && errno != ENOENT) {
         rs_log_error("failed to remove %s: %s", filename, strerror(errno));
-        return -1;
+        return EXIT_IO_ERROR;
     }
 
     ofd = open(filename, O_TRUNC|O_WRONLY|O_CREAT|O_EXCL|O_BINARY, 0666);
     if (ofd == -1) {
         rs_log_error("failed to create %s: %s", filename, strerror(errno));
-        return -1;
+        return EXIT_IO_ERROR;
     }
 
     ret = dcc_r_file_body(ofd, ifd, len);
@@ -246,7 +249,7 @@ int dcc_r_file(int ifd, const char *filename, size_t len)
         rs_log_error("failed to unlink %s after failed transfer: %s",
                      filename, strerror(errno));
     }
-    return -1;
+    return EXIT_IO_ERROR;
 }
 
 
@@ -284,7 +287,7 @@ int dcc_r_file_timed(int ifd, const char *fname, size_t size)
         double secs, rate;
         
         dcc_calc_rate(size, &before, &after, &secs, &rate);
-        rs_log_info("%ld bytes received in %.3fs, rate %.0fkB/s",
+        rs_log_info("%ld bytes received in %.6fs, rate %.0fkB/s",
                     (long) size, secs, rate);
     }
 
