@@ -31,7 +31,6 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <unistd.h>
-#include <assert.h>
 #include <string.h>
 #include <fcntl.h>
 #include <errno.h>
@@ -46,6 +45,7 @@
 
 #include <netdb.h>
 
+#include "types.h"
 #include "distcc.h"
 #include "trace.h"
 #include "io.h"
@@ -53,10 +53,6 @@
 #include "clinet.h"
 #include "util.h"
 
-#ifndef HAVE_IN_PORT_T
-typedef int in_port_t;
-#endif
-
 
 /**
  * @file
@@ -66,53 +62,38 @@ typedef int in_port_t;
  * @todo In error messages, show the name of the relevant host.
  * Should do this even in readx(), etc.
  *
- * @todo Use the new sockets API (optionally?) to support IPv6.  This
- * must be done in a way that will still compile on older machines.
+ * @todo Connect can take a very long time to fail.  We probably need a
+ * userspace timeout on it, e.g. by doing a nonblocking connect and select()
+ * on it.  Check how to do this properly.
  **/
 
 
 
 /**
- * Open a socket to a tcp remote host with the specified port.
- *
- * @todo Don't try for too long to connect. 
+ * @p host and @p port are only here to aid printing debug messages.
  **/
-int dcc_open_socket_out(const char *host, int port, int *p_fd)
+static int dcc_connect_by_addr(const char *host, int port,
+                               struct sockaddr *sa, size_t sa_len,
+                               int *p_fd)
 {
-    int type = SOCK_STREAM;
-    struct sockaddr_in sock_out;
     int fd;
-    struct hostent *hp;
 
-    /* Ignore SIGPIPE; we consistently check error codes and will
-     * see the EPIPE. */
-    dcc_ignore_sigpipe(1);
-
-    fd = socket(PF_INET, type, 0);
+    fd = socket(sa->sa_family, SOCK_STREAM, 0);
+    
     if (fd == -1) {
 	rs_log_error("failed to create socket: %s", strerror(errno));
 	return EXIT_CONNECT_FAILED;
     }
 
-    /* FIXME: warning: gethostbyname() leaks memory.  Use gethostbyname_r instead! */
-    hp = gethostbyname(host);
-    if (!hp) {
-	rs_log_error("unknown host: \"%s\"", host);
-	(void) close(fd);
-	return EXIT_CONNECT_FAILED;
-    }
-
-    memcpy(&sock_out.sin_addr, hp->h_addr, (size_t) hp->h_length);
-    sock_out.sin_port = htons((in_port_t) port);
-    sock_out.sin_family = PF_INET;
-
-    if (connect(fd, (struct sockaddr *) &sock_out, (int) sizeof(sock_out))) {
+    if (connect(fd, sa, sa_len)) {
+        /* TODO: Print raw sockaddr form in some readable way. */
         rs_log_error("failed to connect to %s port %d: %s", host, port, 
                      strerror(errno));
-	(void) close(fd);
+        close(fd);
 	return EXIT_CONNECT_FAILED;
     }
 
+        /* TODO: Print raw sockaddr form in some readable way. */
     rs_trace("client got connection to %s port %d on fd%d", host, port, fd);
 
     *p_fd = fd;
@@ -120,3 +101,72 @@ int dcc_open_socket_out(const char *host, int port, int *p_fd)
 }
 
 
+#if defined(ENABLE_RFC2553)
+
+/**
+ * Open a socket to a tcp remote host with the specified port.
+ *
+ * @todo Don't try for too long to connect. 
+ **/
+int dcc_connect_by_name(const char *host, int port, int *p_fd)
+{
+    struct addrinfo hints;
+    struct addrinfo *res;
+    int error;
+    int ret;
+    char portname[20];
+
+    /* Unfortunately for us, getaddrinfo wants the port (service) as a string */
+    snprintf(portname, sizeof portname, "%d", port);
+
+    memset(&hints, 0, sizeof(hints));
+    /* set-up hints structure */
+    hints.ai_family = PF_UNSPEC;
+    hints.ai_socktype = SOCK_STREAM;
+    error = getaddrinfo(host, portname, &hints, &res);
+    if (error) {
+        rs_log_error("failed to resolve host %s port %d: %s", host, port,
+                     gai_strerror(error));
+        return EXIT_CONNECT_FAILED;
+    }
+
+    /* Try each of the hosts possible addresses. */
+    do {
+        ret = dcc_connect_by_addr(host, port, res->ai_addr, res->ai_addrlen,
+                                  p_fd);
+    } while (ret != 0 && (res = res->ai_next));
+
+    return ret;
+}
+
+
+#else /* not ENABLE_RFC2553 */
+
+/**
+ * Open a socket to a tcp remote host with the specified port.
+ *
+ * @todo Don't try for too long to connect. 
+ **/
+int dcc_connect_by_name(const char *host, int port, int *p_fd)
+{
+    struct sockaddr_in sock_out;
+    struct hostent *hp;
+
+    /* FIXME: "warning: gethostbyname() leaks memory.  Use gethostbyname_r
+     * instead!" (or indeed perhaps use getaddrinfo?) */
+    hp = gethostbyname(host);
+    if (!hp) {
+	rs_log_error("failed to look up host \"%s\": %s", host,
+                     hstrerror(h_errno));
+	return EXIT_CONNECT_FAILED;
+    }
+
+    memcpy(&sock_out.sin_addr, hp->h_addr, (size_t) hp->h_length);
+    sock_out.sin_port = htons((in_port_t) port);
+    sock_out.sin_family = PF_INET;
+
+    return dcc_connect_by_addr(host, port, (struct sockaddr *) &sock_out,
+                               sizeof sock_out, p_fd);
+}
+
+#endif /* not ENABLE_RFC2553 */
