@@ -1,9 +1,8 @@
 /* -*- c-file-style: "java"; indent-tabs-mode: nil; fill-column: 78 -*-
  * 
  * distcc -- A simple distributed compiler system
- * $Header: /data/cvs/distcc/src/cpp.c,v 1.5 2003/02/22 01:41:31 mbp Exp $ 
  *
- * Copyright (C) 2002 by Martin Pool <mbp@samba.org>
+ * Copyright (C) 2002, 2003 by Martin Pool <mbp@samba.org>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -43,6 +42,7 @@
 #include "exec.h"
 #include "tempfile.h"
 #include "cpp.h"
+#include "filename.h"
 
 
 /**
@@ -65,10 +65,16 @@ int dcc_cpp_maybe(char **argv, char *input_fname, char **cpp_fname,
 {
     char **cpp_argv;
     int ret;
+    char *input_exten;
+    const char *output_exten;
 
     *cpp_pid = 0;
     
     if (dcc_is_preprocessed(input_fname)) {
+        /* TODO: Perhaps also consider the option that says not to use cpp.
+         * Would anyone do that? */
+        rs_trace("input is already preprocessed");
+        
         /* already preprocessed, great. */
         if (!(*cpp_fname = strdup(input_fname))) {
             rs_fatal("couldn't duplicate string");
@@ -76,7 +82,14 @@ int dcc_cpp_maybe(char **argv, char *input_fname, char **cpp_fname,
         return 0;
     }
 
-    *cpp_fname = dcc_make_tmpnam("cppout", ".i");
+    /* TODO: Perhaps use .ii for C++ files? */
+    /* TODO: Really we ought to centralize determination of file type. */
+    /* TODO: Does this also affect the server temporary file?  It ought to. */
+
+    input_exten = dcc_find_extension(input_fname);
+    output_exten = dcc_preproc_exten(input_exten);
+    *cpp_fname = dcc_make_tmpnam("cppout", output_exten);
+
 
     /* FIXME: Stripping -o fixes some, but not all problems with -MD.
      *
