@@ -1,7 +1,6 @@
 /* -*- c-file-style: "java"; indent-tabs-mode: nil; fill-column: 78; -*-
  * 
  * distcc -- A simple distributed compiler system
- * $Header: /data/cvs/distcc/src/daemon.c,v 1.81 2003/01/27 14:02:00 mbp Exp $ 
  *
  * Copyright (C) 2002, 2003 by Martin Pool <mbp@samba.org>
  *
@@ -57,7 +56,7 @@
 #include "trace.h"
 #include "io.h"
 #include "util.h"
-#include "opt.h"
+#include "dopt.h"
 #include "setuid.h"
 #include "srvnet.h"
 
@@ -67,7 +66,49 @@ char const *rs_program_name = "distccd";
 
 
 static int dcc_inetd_server(void);
-static void dccd_setup_log(void);
+static void dcc_setup_real_log(void);
+
+
+/**
+ * Errors during startup (e.g. bad options) need to be reported somewhere,
+ * although we have not yet parsed the options to work out where the user
+ * wants them.
+ *
+ * In inetd mode, we can't write to stderr because that will corrupt the
+ * stream, so if it looks like stderr is a socket we go to syslog instead.
+ **/
+static int dcc_setup_startup_log(void)
+{
+    rs_trace_set_level(RS_LOG_INFO);
+    if (!is_a_socket(STDERR_FILENO)) {
+        rs_add_logger(rs_logger_file, RS_LOG_DEBUG, 0, STDERR_FILENO);
+    } else {
+        openlog("distccd", LOG_PID, LOG_DAEMON);
+        rs_add_logger(rs_logger_syslog, RS_LOG_DEBUG, NULL, 0);
+    }
+
+    return 0;
+}
+
+
+static int dcc_should_be_inetd(void)
+{
+    /* Work out if we ought to serve stdin or be a standalone daemon */
+    if (opt_inetd_mode)
+        return 1;
+    else if (opt_daemon_mode)
+        return 0;
+    else if (is_a_socket(STDIN_FILENO)) {
+        rs_log_info("stdin is socket; assuming --inetd mode");
+        return 1;
+    } else if (isatty(STDIN_FILENO)) {
+        rs_log_info("stdin is a tty; assuming --daemon mode");
+        return 0;
+    } else {
+        rs_log_info("stdin is neither a tty nor a socket; assuming --daemon mode");
+        return 0;
+    }
+}
 
 
 /**
@@ -76,57 +117,73 @@ static void dccd_setup_log(void);
  **/
 int main(int argc, char *argv[])
 {
+    const char *requested_path;
     int ret;
 
-    /* Any errors during startup (e.g. bad options) ought to go to stderr.
-     * Once we're up and running, we use just the specified log file. */
-    rs_trace_set_level(RS_LOG_INFO);
-    rs_add_logger(rs_logger_file, RS_LOG_DEBUG, 0, STDERR_FILENO);
+    dcc_setup_startup_log();
 
     if (distccd_parse_options(argc, (const char **) argv))
         dcc_exit(EXIT_DISTCC_FAILED);
 
+    if ((ret = dcc_set_lifetime()) != 0)
+        dcc_exit(ret);
+
     if ((ret = dcc_discard_root()) != 0)
         dcc_exit(ret);
 
     /* Discard privileges before opening log so that if it's created, it has
      * the right ownership. */
-    dccd_setup_log();
+    dcc_setup_real_log();
+     
+    if ((requested_path = getenv("DISTCCD_PATH")) != NULL)
+        ret = dcc_set_path(requested_path);
+    else
+        ret = dcc_trim_path(NULL);
+    if (ret)
+        dcc_exit(ret);
 
-    /* This test might need to be a bit more complex when we want to
-     * support ssh and stuff like that.  Perhaps it would be better to
-     * always serve stdin, and then complain if it's a terminal? */
-    if (opt_inetd_mode) {
+    if (dcc_should_be_inetd())
         ret = dcc_inetd_server();
-    } else if (opt_daemon_mode) {
+    else
         ret = dcc_standalone_server();
-    } else if (is_a_socket(STDIN_FILENO)) {
-        rs_log_info("stdin is socket; assuming --inetd mode");
-        ret = dcc_inetd_server();
-    } else if (isatty(STDIN_FILENO)) {
-        rs_log_info("stdin is a tty; assuming --daemon mode");
-        ret = dcc_exit(dcc_standalone_server());
-    } else {
-        rs_log_info("stdin is neither a tty nor a socket; assuming --daemon mode");
-        ret = dcc_standalone_server();
-    }
+    
     dcc_exit(ret);
 }
 
 
-static void dccd_setup_log(void)
+/**
+ * If a --lifetime options was specified, set up a timer that will kill the
+ * daemon when it expires.
+ **/
+int dcc_set_lifetime(void)
+{
+    if (opt_lifetime) {
+        alarm(opt_lifetime);
+/*         rs_trace("set alarm for %+d seconds", opt_lifetime); */
+    }
+    return 0;
+}
+
+
+/**
+ * Set log to the final destination after options have been read.
+ **/
+static void dcc_setup_real_log(void)
 {
     int fd;
-    
-    /* We start off tracing to stderr, so that bad options will be
-     * logged to an obvious place.  This can be overridden by the
-     * --log-file= option, handled in dopt.c; otherwise we switch to
-     * syslog. */
 
-    if (opt_log_stderr)
+    /* Even in inetd mode, we might want to log to stderr, because that will
+     * work OK for ssh connections. */
+    
+    if (opt_log_stderr) {
+        rs_remove_all_loggers();
+        rs_add_logger(rs_logger_file, RS_LOG_DEBUG, 0, STDERR_FILENO);
         return;
+    }
     
     if (arg_log_file) {
+        /* Don't remove loggers yet, in case this fails and needs to go to the
+         * default. */
         if ((fd = open(arg_log_file, O_CREAT|O_APPEND|O_WRONLY, 0666)) == -1) {
             rs_log_error("failed to open %s: %s", arg_log_file,
                          strerror(errno));
@@ -138,7 +195,6 @@ static void dccd_setup_log(void)
         }
     }
     
-    openlog("distccd", LOG_PID, LOG_DAEMON);
     rs_remove_all_loggers();
     rs_add_logger(rs_logger_syslog, RS_LOG_DEBUG, NULL, 0);
 }
