@@ -60,6 +60,7 @@
 #include "lock.h"
 #include "cpp.h"
 #include "clirpc.h"
+#include "backoff.h"
 
 
 /* Name of this program, for trace.c */
@@ -106,21 +107,27 @@ static void dcc_show_usage(void)
 "   --version                  show version and exit\n"
 "\n"
 "Environment variables:\n"
-"   DISTCC_HOSTS=\"HOST ...\"\n"
-"            list of volunteer hosts, should include localhost\n"
 "   DISTCC_VERBOSE=1           give debug messages\n"
 "   DISTCC_LOG=LOGFILE         send messages here, not stderr\n"
 "   DISTCC_TCP_CORK=0          disable TCP corks\n"
+"   DISTCC_SSH=cmd             command to run to open SSH connections\n"
+"\n"
+"Server specification:\n"
+"A list of servers is taken from the environment variable $DISTCC_HOSTS, or\n"
+"the file $HOME/.distcc/hosts, or the file %s/distcc/hosts.\n"
+"Each host can be given in any of these forms:\n"
 "\n"
-"Host specifications:\n"
 "   localhost                  run in place\n"
 "   HOST                       TCP connection, port %d\n"
 "   HOST:PORT                  TCP connection, specified port\n"
+"   @HOST                      SSH connection\n"
+"   USER@HOST                  SSH connection to specified host\n"
 "\n"
 "distcc distributes compilation jobs across volunteer machines running\n"
 "distccd.  Jobs that cannot be distributed, such as linking or \n"
 "preprocessing are run locally.  distcc should be used with make's -jN\n"
 "option to execute in parallel on several machines.\n",
+    SYSCONFDIR,
     DISTCC_DEFAULT_PORT);
 }
 
@@ -160,33 +167,62 @@ static void dcc_show_usage(void)
 static int dcc_compile_remote(char **argv, 
                               char *cpp_fname, char *output_fname,
                               pid_t cpp_pid,
-                              const struct dcc_hostdef *host,
+                              struct dcc_hostdef *host,
                               int *status,
                               int xmit_lock_fd)
 {
-    int net_fd;
+    int to_net_fd, from_net_fd;
     int ret;
+    pid_t ssh_pid;
+
+    /* For ssh support, we need to allow for separate fds writing to and
+     * reading from the network, because our connection to the ssh client may
+     * be over pipes, which are one-way connections. */
 
     *status = 0;
 
-    dcc_note_execution(host->hostname, argv);
-    if ((ret = dcc_open_socket_out(host->hostname, host->port, &net_fd)) != 0)
-        return ret;
+    dcc_note_execution(host, argv);
+    if (host->mode == DCC_MODE_TCP) {
+        if ((ret = dcc_connect_by_name(host->hostname, host->port,
+                                       &to_net_fd)) != 0)
+            return ret;
+        from_net_fd = to_net_fd;
+    } else if (host->mode == DCC_MODE_SSH) {
+        /* FIXME: This breaks badly if we don't have socketpair, because the
+         * two fds won't be the same.  Need to split everything from here on
+         * in to use separate in/out fds.
+         */
+        if ((ret = dcc_ssh_connect(NULL, host->user, host->hostname,
+                                   host->ssh_command,
+                                   &from_net_fd, &to_net_fd,
+                                   &ssh_pid)))
+            return ret;
+    } else {
+        rs_fatal("impossible host mode");
+    }
 
     /* This waits for cpp and puts its status in *status.  If cpp failed, then
      * the connection will have been dropped and we need not bother trying to
      * get any response from the server. */
-    ret = dcc_send_job_corked(net_fd, argv, cpp_pid, status, cpp_fname);
+    ret = dcc_send_job_corked(to_net_fd, argv, cpp_pid, status, cpp_fname);
 
     dcc_unlock(xmit_lock_fd);
-    
+
+    if (to_net_fd != from_net_fd)
+        dcc_close(to_net_fd);
+
     if (ret == 0 && *status == 0) {
-        ret = dcc_retrieve_results(net_fd, status, output_fname);
+        ret = dcc_retrieve_results(from_net_fd, status, output_fname);
     }
 
     /* Close socket so that the server can terminate, rather than
      * making it wait until we've finished our work. */
-    dcc_close(net_fd);
+    dcc_close(from_net_fd);
+
+    /* XXX: Should we collect the SSH child?  It would be nice to get a clear
+     * error if it failed, but we don't know when it will finish cleaning up
+     * and I don't want to wait.  (In theory it should write to stderr if
+     * something goes wrong.) */
 
     return ret;
 }
@@ -214,9 +250,8 @@ static int dcc_compile_local(char *argv[])
     int ret;
     int status;
     long u_us, s_us;
-    const char *buildhost = "localhost";
 
-    dcc_note_execution(buildhost, argv);
+    dcc_note_execution(dcc_hostdef_local, argv);
 
     /* We don't do any redirection of file descriptors when running locally,
      * so if for example cpp is being used in a pipeline we should be fine. */
@@ -227,7 +262,7 @@ static int dcc_compile_local(char *argv[])
         return ret;
 
     dcc_report_rusage(argv[0], u_us, s_us);
-    return dcc_critique_status(status, "compile", dcc_gethostname());
+    return dcc_critique_status(status, "compile", dcc_hostdef_local, 1);
 }
 
 
@@ -253,7 +288,7 @@ static int dcc_compile_local(char *argv[])
  * the compiler itself fails.  If either the compiler or preprocessor fails,
  * @p status is guaranteed to hold a failure value.
  **/
-static int dcc_build_somewhere(char *argv[], int *status)
+static int dcc_build_somewhere(char *argv[], int sg_level, int *status)
 {
     char *input_fname, *output_fname, *cpp_fname;
     char **argv_stripped;
@@ -262,6 +297,9 @@ static int dcc_build_somewhere(char *argv[], int *status)
     int ret;
     struct dcc_hostdef *host = NULL;
 
+    if (sg_level)
+        goto run_local;
+
     /* TODO: Perhaps tidy up these gotos. */
 
     if (dcc_scan_args(argv, &input_fname, &output_fname, &argv) != 0) {
@@ -270,7 +308,7 @@ static int dcc_build_somewhere(char *argv[], int *status)
         goto lock_local;
     }
 
-    if ((ret = dcc_pick_host_from_env(&host, &xmit_lock_fd, &cpu_lock_fd)) != 0) {
+    if ((ret = dcc_pick_host_from_list(&host, &xmit_lock_fd, &cpu_lock_fd)) != 0) {
         /* Doesn't happen at the moment: all failures are masked by
            returning localhost. */
         goto fallback;
@@ -292,14 +330,23 @@ static int dcc_build_somewhere(char *argv[], int *status)
         goto fallback;
     }
 
+    dcc_enjoyed_host(host);
+    
     dcc_unlock(cpu_lock_fd);
-        
-    return dcc_critique_status(*status, "compile", host->hostname);
+
+    return dcc_critique_status(*status, "compile", host, 1);
 
   fallback:
+    if (host)
+        dcc_disliked_host(host);
+    
+    if (!dcc_getenv_bool("DISTCC_FALLBACK", 1)) {
+        rs_log_warning("failed to distribute and fallbacks are disabled");
+        return ret;
+    }
+    
     /* "You guys are so lazy!  Do I have to do all the work myself??" */
-    rs_log_warning("failed to distribute to \"%s\", running locally instead",
-                   host && host->hostname ? host->hostname : "(unknown)");
+    rs_log_warning("failed to distribute, running locally instead");
 
   lock_local:
 
@@ -311,8 +358,7 @@ static int dcc_build_somewhere(char *argv[], int *status)
 
 
 /**
- * Send trace to append to the file specified by DISTCC_LOG.  If
- * that's something you didn't want to write to, tough.
+ * Send trace to append to the file specified by DISTCC_LOG.  
  *
  * The exact setting of log level is a little strange, but for a good
  * reason: if you ask for verbose, you get everything.  Otherwise, if
@@ -323,18 +369,25 @@ static void dcc_set_trace_from_env(void)
 {
     const char *logfile;
 
-    if ((logfile = getenv("DISTCC_LOG"))) {
+    if ((logfile = getenv("DISTCC_LOG")) && logfile[0]) {
         int fd;
 
         rs_trace_set_level(RS_LOG_INFO);
 
         fd = open(logfile, O_WRONLY|O_APPEND|O_CREAT, 0666);
         if (fd == -1) {
+            /* use stderr instead */
+            int save_errno = errno;
+            
+            rs_trace_set_level(RS_LOG_WARNING);
+            rs_add_logger(rs_logger_file, RS_LOG_DEBUG, NULL, STDERR_FILENO);
+
             rs_log_error("failed to open logfile %s: %s",
-                         logfile, strerror(errno));
+                         logfile, strerror(save_errno));
+        } else {
+            rs_add_logger(rs_logger_file, RS_LOG_DEBUG, NULL, fd);
+            rs_trace_set_level(RS_LOG_INFO);
         }
-        rs_add_logger(rs_logger_file, RS_LOG_DEBUG, NULL, fd);
-        rs_trace_set_level(RS_LOG_INFO);
     } else {
         rs_trace_set_level(RS_LOG_WARNING);
         rs_add_logger(rs_logger_file, RS_LOG_DEBUG, NULL, STDERR_FILENO);
@@ -346,6 +399,21 @@ static void dcc_set_trace_from_env(void)
 }
 
 
+
+static void dcc_get_my_basename(char *argv[], char **myname)
+{
+    char *progname;
+    
+    /* Find our basename */
+    if ((progname = strrchr(argv[0], '/')) != NULL)
+        progname++;
+    else
+        progname = argv[0];
+
+    *myname = progname;
+}
+
+
 /**
  * distcc client entry point.
  *
@@ -356,24 +424,51 @@ static void dcc_set_trace_from_env(void)
  **/
 int main(int argc, char **argv)
 {
-    int status;
-    char **compiler_args;
+    int status, sg_level, tweaked_path = 0;
+    char **compiler_args, *progname;
+    int ret;
 
     atexit(dcc_cleanup_tempfiles);
 
     dcc_set_trace_from_env();
 
-    if (argc <= 1 || !strcmp(argv[1], "--help")) {
-        dcc_show_usage();
-        exit(0);
-    } else if (!strcmp(argv[1], "--version")) {
-        dcc_show_version("distcc");
-        exit(0);
+    dcc_trace_version();
+    
+    dcc_get_my_basename(argv, &progname);
+
+    /* Ignore SIGPIPE; we consistently check error codes and will
+     * see the EPIPE. */
+    dcc_ignore_sigpipe(1);
+
+    sg_level = dcc_recursion_safeguard();
+
+    if (strstr(progname, "distcc") != NULL) {
+        /* Either "distcc -c hello.c" or "distcc gcc -c hello.c" */
+        if (argc <= 1 || !strcmp(argv[1], "--help")) {
+            dcc_show_usage();
+            exit(0);
+        }
+        if (!strcmp(argv[1], "--version")) {
+            dcc_show_version("distcc");
+            exit(0);
+        }
+        dcc_find_compiler(argv, &compiler_args);
+        dcc_get_my_basename(compiler_args, &progname);
+        if ((ret = dcc_trim_path(progname)) != 0)
+            dcc_exit(ret);
+    } else {
+        /* Invoked as "cc -c hello.c", with masqueraded path */
+        if ((ret = dcc_support_masquerade(argv, progname, &tweaked_path)) != 0)
+            dcc_exit(ret);
+        
+        dcc_shallowcopy_argv(argv, &compiler_args, 0);
+        compiler_args[0] = progname;
     }
 
-    dcc_recursion_safeguard();
+    if (sg_level - tweaked_path > 0) {
+        rs_log_crit("distcc seems to have invoked itself recursively!");
+        dcc_exit(EXIT_RECURSION);
+    }
 
-    dcc_find_compiler(argv, &compiler_args);
-
-    dcc_exit(dcc_build_somewhere(compiler_args, &status));
+    dcc_exit(dcc_build_somewhere(compiler_args, sg_level, &status));
 }
