@@ -78,6 +78,20 @@ char *dcc_argv_tostr(char **a);
 
 int dcc_argv_len(char **a);
 
+
+/* A macro so that we get the right __FUNCTION__ in the trace message.
+ *
+ * We condition on rs_trace_enabled so that we don't do the to-string
+ * conversion unless the user will actually see the result.  */
+#define dcc_trace_argv(_message, _argv)         \
+    if (rs_trace_enabled()) {                   \
+        char *_astr;                            \
+        _astr = dcc_argv_tostr(_argv);          \
+        rs_trace("%s: %s", _message, _astr);    \
+        free(_astr);                            \
+    } else while (0)
+
+
 /* dopt.c */
 extern struct dcc_allow_list *opt_allowed;
 int distccd_parse_options(int argc, const char *argv[]);
@@ -86,15 +100,22 @@ int distccd_parse_options(int argc, const char *argv[]);
 int dcc_accept_job(int fd);
 
 /* help.c */
+int dcc_trace_version(void);
 int dcc_show_copyright(void);
 int dcc_show_version(const char *prog);
 
 /* daemon.c */
 void dcc_server_child(int) NORETURN;
 int dcc_refuse_root(void);
+int dcc_set_lifetime(void);
+
+/* dsignal.c */
+void dcc_catch_signals(void);
+void dcc_ignore_sighup(void);
 
 /* dparent.c */
 int dcc_standalone_server(void);
+void dcc_remove_pid(void);
 
 
 /* bulk.c */
@@ -108,5 +129,22 @@ int dcc_parse_hosts(const char *where,
                     struct dcc_hostdef **ret_list,
                     int *ret_nhosts);
 
+/* ncpu.c */
+int dcc_ncpus(int *);
+
+/* ssh.c */
+int dcc_ssh_connect(char *ssh_cmd, char *user,
+                    char *machine, char *path,
+                    int *f_in, int *f_out,
+                    pid_t *ssh_pid);
+
+/* safeguard.c */
+int dcc_increment_safeguard(void);
+int dcc_recursion_safeguard(void);
+
+/* climasq.c */
+int dcc_support_masquerade(char *argv[], char *progname, int *);
+
+
 #define DISTCC_DEFAULT_PORT 3632
 
