@@ -43,21 +43,12 @@
 
 
 
-/**
- * This environment variable is set to guard against distcc
- * accidentally recursively invoking itself, thinking it's the real
- * compiler.
- **/
-static const char *dcc_safeguard_name = "_DISTCC_SAFEGUARD";
-static const char *dcc_safeguard_set = "_DISTCC_SAFEGUARD=1";
-
 
 #include "config.h"
 
 #include <stdio.h>
 #include <stdlib.h>
 #include <unistd.h>
-#include <assert.h>
 #include <string.h>
 #include <fcntl.h>
 #include <errno.h>
@@ -76,18 +67,21 @@ static const char *dcc_safeguard_set = "_DISTCC_SAFEGUARD=1";
 #include "exitcode.h"
 #include "exec.h"
 #include "lock.h"
+#include "hosts.h"
+
 
 /* You can't have core files on Windows. */
 #ifndef WCOREDUMP
 #  define WCOREDUMP(status) 0
 #endif
 
-void dcc_note_execution(const char *hostname, char **argv)
+void dcc_note_execution(struct dcc_hostdef *host, char **argv)
 {
     char *astr;
 
     astr = dcc_argv_tostr(argv);
-    rs_log(RS_LOG_INFO|RS_LOG_NONAME, "exec on %s: %s", hostname, astr);
+    rs_log(RS_LOG_INFO|RS_LOG_NONAME, "exec on %s: %s",
+           host->hostdef_string, astr);
     free(astr);
 }
 
@@ -131,7 +125,7 @@ int dcc_remove_if_exists(const char *fname)
     if (unlink(fname) && errno != ENOENT) {
         rs_log_warning("failed to unlink %s: %s", fname,
                        strerror(errno));
-        return -1;
+        return EXIT_IO_ERROR;
     }
     return 0;
 }
@@ -154,15 +148,6 @@ void dcc_execvp(char **argv)
 }
 
 
-void dcc_recursion_safeguard(void)
-{
-    if (getenv(dcc_safeguard_name)) {
-        rs_log_crit("distcc seems to have invoked itself recursively!");
-        dcc_exit(EXIT_RECURSION);
-    }
-}
-
-
 
 /**
  * Called inside the newly-spawned child process to execute a command.
@@ -192,10 +177,8 @@ static int dcc_inside_child(char **argv,
     if ((ret = dcc_ignore_sigpipe(0)))    /* set handler back to default */
         return ret;
 
-    if ((putenv(strdup(dcc_safeguard_set)) == -1)) {
-        rs_log_error("setenv failed");
-        /* and continue */
-    }
+    /* Ignore failure */
+    dcc_increment_safeguard();
 
     /* do this last, so that any errors from previous operations are
      * visible */
@@ -209,9 +192,10 @@ static int dcc_inside_child(char **argv,
 void dcc_setpgid(pid_t pid, pid_t pgid)
 {
     if (setpgid(pid, pgid) == 0)
-        rs_trace("setpgid(%d, %d)", pid, pgid);
+        rs_trace("setpgid(%ld, %ld)", (long) pid, (long) pgid);
     else
-        rs_trace("setpgid(%d, %d) failed: %s", pid, pgid, strerror(errno));
+        rs_trace("setpgid(%ld, %ld) failed: %s",
+                 (long) pid, (long) pgid, strerror(errno));
     
     /* Continue anyhow i guess.  Failure seems unlikely.  */
 }
@@ -235,12 +219,12 @@ int dcc_spawn_child(char **argv, pid_t *pidptr,
 {
     pid_t pid;
 
-    rs_trace("forking to execute %s", dcc_argv_tostr(argv));
+    dcc_trace_argv("forking to execute", argv);
     
     pid = fork();
     if (pid == -1) {
         rs_log_error("failed to fork: %s", strerror(errno));
-        return -1;
+        return EXIT_OUT_OF_MEMORY; /* probably */
     } else if (pid == 0) {
         exit(dcc_inside_child(argv, stdin_file, stdout_file, stderr_file));
         /* !! NEVER RETURN FROM HERE !! */
@@ -297,7 +281,8 @@ int dcc_collect_child(pid_t pid, int *wait_status, long *u_us, long *s_us)
         if ((ret_pid = sys_wait4(pid, wait_status, 0, &ru)) != -1) {
             /* This is not the main user-visible message, that comes from
              * critique_status(). */
-            rs_trace("child %d terminated with status %#x", ret_pid, *wait_status);
+            rs_trace("child %ld terminated with status %#x",
+                     (long) ret_pid, *wait_status);
             *u_us = ru.ru_utime.tv_sec * 1000000 + ru.ru_utime.tv_usec;
             *s_us = ru.ru_stime.tv_sec * 1000000 + ru.ru_stime.tv_usec;
             return 0;
@@ -306,7 +291,7 @@ int dcc_collect_child(pid_t pid, int *wait_status, long *u_us, long *s_us)
             continue;
         } else {
             rs_log_error("sys_wait4(pid=%d) borked: %s", (int) pid, strerror(errno));
-            return -1;
+            return EXIT_DISTCC_FAILED;
         }
     }
 }
@@ -336,25 +321,38 @@ int dcc_report_rusage(const char *command,
  * signal; otherwise the command's exit code.
  **/
 int dcc_critique_status(int s, const char *command,
-                        const char *hostname)
+                        struct dcc_hostdef *host,
+                        int verbose)
 {
+    int logmode;
+
+    /* verbose mode is only used for executions that the user is likely to
+     * particularly need to know about */
+    if (verbose)
+        logmode = RS_LOG_ERR | RS_LOG_NONAME;
+    else
+        logmode = RS_LOG_INFO;
+    
     if (WIFSIGNALED(s)) {
-        rs_log_error("%s on %s died with signal %d%s",
-                     command, hostname,
-                     WTERMSIG(s),
-                     WCOREDUMP(s) ? " (core dumped)" : "");
+        rs_log(logmode,
+               "%s on %s died with signal %d%s",
+               command, host->hostdef_string,
+               WTERMSIG(s),
+               WCOREDUMP(s) ? " (core dumped)" : "");
         /* Unix convention is to return 128+signal when a subprocess crashes. */
         return 128 + WTERMSIG(s);
     } else if (WEXITSTATUS(s)) {
-        /* Only a NOTICE because having compilers fail is not really
-         * unexpected and we don't want to flood the client's error
-         * stream. */
-        rs_log_notice("%s on %s failed with exit code %d",
-                     command, hostname, WEXITSTATUS(s));
+        /* This is a tough call; we don't really want to clutter the client's
+         * error stream, but if we don't say where the compilation failed then
+         * people may find it hard to work things out. */
+
+        rs_log(logmode,
+               "%s on %s failed with exit code %d",
+               command, host->hostdef_string, WEXITSTATUS(s));
         return WEXITSTATUS(s);
     } else {
-        rs_log(RS_LOG_INFO|RS_LOG_NONAME,
-               "%s on %s completed ok", command, hostname);
+        rs_log(RS_LOG_INFO,
+               "%s on %s completed ok", command, host->hostdef_string);
         return 0;
     }
 }
