@@ -1,9 +1,8 @@
 /* -*- c-file-style: "java"; indent-tabs-mode: nil -*- 
  *
  * distcc -- A simple distributed compiler system
- * $Header: /data/cvs/distcc/src/filename.c,v 1.7 2003/02/22 01:41:31 mbp Exp $ 
  *
- * Copyright (C) 2002 by Martin Pool <mbp@samba.org>
+ * Copyright (C) 2002, 2003 by Martin Pool <mbp@samba.org>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -27,7 +26,6 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <unistd.h>
-#include <assert.h>
 #include <string.h>
 #include <errno.h>
 
@@ -40,12 +38,23 @@
 #include "filename.h"
 
 
+/**
+ * @file
+ * Everything we know about C filenames.
+ **/
+
+
 /*
  * NOTE: As of 0.10, .s and .S files are never distributed, because
  * they might contain '.include' pseudo-operations, which are resolved
  * by the assembler.
  */
 
+/**
+ * XXX: The non-free Intel C compiler (icc) may have trouble with
+ * preprocessed C++ source files called .ii
+ **/
+
 
 /**
  * Return a pointer to the extension, including the dot, or NULL.
@@ -75,11 +84,11 @@ static int dcc_set_file_extension(const char *sfile,
     dot = dcc_find_extension((char *) o);
     if (!dot) {
         rs_log_error("couldn't find extension in \"%s\"", o);
-        return -1;
+        return EXIT_DISTCC_FAILED;
     }
     if (strlen(dot) < strlen(new_ext)) {
         rs_log_error("not enough space for new extension");
-        return -1;
+        return EXIT_DISTCC_FAILED;
     }
     strcpy(dot, new_ext);
     *ofile = o;
@@ -88,6 +97,19 @@ static int dcc_set_file_extension(const char *sfile,
 }
 
 
+/*
+ * Apple extensions:
+ * file.mm, file.M 
+ * Objective-C++ source code which must be preprocessed. (APPLE ONLY) 
+ *
+ * file.mii Objective-C++ source code which should not be
+ * preprocessed. (APPLE ONLY)
+ *
+ * http://developer.apple.com/techpubs/macosx/DeveloperTools/gcc3/gcc/Overall-Options.html
+ */
+    
+
+
 /**
  * If you preprocessed a file with extension @p e, what would you get?
  *
@@ -108,6 +130,11 @@ const char * dcc_preproc_exten(const char *e)
                || !strcmp(e, "cp") || !strcmp(e, "c++")
                || !strcmp(e, "C") || !strcmp(e, "ii")) {
         return ".ii";
+    } else if(!strcmp(e,"mi") || !strcmp(e, "m")) {
+        return ".mi";
+    } else if(!strcmp(e,"mii") || !strcmp(e,"mm")
+                || !strcmp(e,"M")) {
+        return ".mii";
     } else if (!strcasecmp(e, "s")) {
         return ".s";
     } else {
@@ -116,6 +143,10 @@ const char * dcc_preproc_exten(const char *e)
 }
 
 
+/**
+ * Does the extension of this file indicate that it is already
+ * preprocessed?
+ **/
 int dcc_is_preprocessed(const char *sfile)
 {
     const char *dot, *ext;
@@ -133,6 +164,9 @@ int dcc_is_preprocessed(const char *sfile)
     case 'i':
         return !strcmp(ext, "i")
             || !strcmp(ext, "ii");
+    case 'm':
+        return !strcmp(ext, "mi")
+            || !strcmp(ext, "mii");
     default:
         return 0;
     }
@@ -166,6 +200,13 @@ int dcc_is_source(const char *sfile)
             || !strcmp(ext, "c++");
     case 'C':
         return !strcmp(ext, "C");
+    case 'm':
+        return !strcmp(ext,"m")
+            || !strcmp(ext,"mm")
+            || !strcmp(ext,"mi")
+            || !strcmp(ext,"mii");
+    case 'M':
+        return !strcmp(ext, "M");
 #ifdef ENABLE_REMOTE_ASSEMBLE
     case 's':
         return !strcmp(ext, "s");
@@ -175,9 +216,6 @@ int dcc_is_source(const char *sfile)
     default:
         return 0;
     }
-
-    /* XXX: Also .m is Objective-C, but I don't know if that will work
-     * so for the moment we always do it locally. -- mbp */
 }
 
 
@@ -204,7 +242,7 @@ int dcc_output_from_source(const char *sfile,
         sfile = slash+1;
     if ((l = strlen(sfile)) < 3) {
         rs_log_error("source file %s is bogus", sfile);
-        return -1;
+        return EXIT_DISTCC_FAILED;
     }
 
     return dcc_set_file_extension(sfile, out_extn, ofile);
