@@ -20,6 +20,10 @@
  * USA
  */
 
+                    /*
+                     * Tell me what you need, and I'll tell you how to 
+                     * get along without it.                -- Dilbert
+                     */
 
 /**
  * @file
@@ -33,7 +37,6 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <unistd.h>
-#include <assert.h>
 #include <string.h>
 #include <errno.h>
 
@@ -41,6 +44,15 @@
 #include "trace.h"
 
 
+int dcc_trace_version(void)
+{
+    rs_trace("%s %s %s; built %s %s",
+             rs_program_name, PACKAGE_VERSION, GNU_HOST,
+             __DATE__, __TIME__);
+    return 0;
+}
+
+
 int dcc_show_version(const char *prog)
 {
     /* The "built" message is the time this file was built, which may not be
