@@ -59,12 +59,28 @@
  * URL-like schemes.
  */
 
+
+/*
+       Alexandre Oliva writes
+
+        I take this opportunity to plead people to consider such issues when
+        proposing additional syntax for DISTCC_HOSTS: if it was possible to
+        handle DISTCC_HOSTS as a single shell word (perhaps after turning
+        blanks into say commas), without the risk of any shell active
+        characters such as {, }, ~, $, quotes getting in the way, outputting
+        distcc commands that override DISTCC_HOSTS would be far
+        simpler.
+
+  TODO: Perhaps entries in the host list that "look like files" (start
+    with '/' or '~') should be read in as files?  This could even be
+    recursive.
+*/
+
 #include "config.h"
 
 #include <stdio.h>
 #include <stdlib.h>
 #include <unistd.h>
-#include <assert.h>
 #include <string.h>
 #include <errno.h>
 #include <time.h>
@@ -78,85 +94,78 @@
 
 const int dcc_default_port = DISTCC_DEFAULT_PORT;
 
-/* TODO: Write and use a test harness for this. */
 
-/* XXX: Where is the right place to handle multi-A-records?  Should
- * they expand into multiple host definitions, or should they be
- * handled later on when locking?
+#ifndef HAVE_STRNDUP
+/**
+ * Copy at most @p size characters from @p src, plus a terminating nul.
  *
- * ssh is an interesting case because we probably want to open the
- * connection using the hostname, so that the ssh config's "Host"
- * sections can have the proper effect.
- *
- * Alternatively perhaps it would be better to avoid multi-A records
- * altogether and just use SRV records.  Simpler in a way...
- *
- * For every hostname occurring in the list, we need to check if it
- * has SRV records, and if so use that instead.  We need to handle the
- * special SRV record which says "canonically not available", by
- * ignoring that host.  This can be done for both SSH and TCP.
- * (Perhaps for SSH it needs to be "distcc.ssh.host.foo.com" rather
- * than "tcp"?)
- *
- * Sometimes people use multi A records for machines with several
- * routeable interfaces.  In that case it would be bad to assume the
- * machine can run multiple jobs, and it is better to let the resolver
- * work out which address to use.
- */
+ * Really this needs to be in util.c, but it's only used here.
+ **/
+static char *strndup(const char *src, size_t size)
+{
+    char *dst;
 
+    dst = malloc(size + 1);
+    if (dst == NULL)
+        return NULL;
+    strncpy(dst, src, size);
+    dst[size] = '\0';
 
+    return dst;
+}
+#endif
 
 /**
- * @post *ret_nhosts>0 if the function succeeded.
+ * Get a list of hosts to use.
  *
- * @post There are @p *ret_nhosts entries in the list at @p *ret_list.
- *
- * @return -1 on error (including no hosts defined), or 0
+ * Hosts are taken from DISTCC_HOSTS, if that exists.  Otherwise, they are
+ * taken from ~/.distcc/hosts, if that exists.  Otherwise, they are taken from
+ * ${sysconfdir}/distcc/hosts, if that exists.  Otherwise, we fail.
  **/
-int dcc_parse_hosts_env(struct dcc_hostdef **ret_list,
-                        int *ret_nhosts)
+int dcc_get_hostlist(struct dcc_hostdef **ret_list,
+                     int *ret_nhosts)
 {
-    char *where;
+    char *env;
+    char *path;
+    char *home;
+    int ret;
 
-    where = getenv("DISTCC_HOSTS");
-    if (!where) {
-        rs_log_warning("$DISTCC_HOSTS is not defined; can't distribute work");
-        return EXIT_BAD_HOSTSPEC;
+    if ((env = getenv("DISTCC_HOSTS")) != NULL) {
+        rs_trace("read hosts from environment");
+        return dcc_parse_hosts(env, ret_list, ret_nhosts);
     }
 
-    return dcc_parse_hosts(where, ret_list, ret_nhosts);
-}
-
-
-/**
- * Duplicate the part of the string @p psrc up to a character in @p sep
- * (or end of string), storing the result in @p pdst.  @p psrc is updated to
- * point to the terminator.  (If the terminator is not found it will
- * therefore point to \0.
- *
- * If there is no more string, then @p pdst is instead set to NULL, no
- * memory is allocated, and @p psrc is not advanced.
- **/
-static int dcc_dup_part(const char **psrc, char **pdst, const char *sep)
-{
-    int len;
-
-    len = strcspn(*psrc, sep);
-    if (len == 0) {
-        *pdst = NULL;
-    } else {    
-        if (!(*pdst = malloc(len + 1))) {
-            rs_log_error("failed to allocate string duplicate: %d", (int) len);
-            return EXIT_OUT_OF_MEMORY;
+    /* ~/.distcc/hosts */
+    if ((home = getenv("HOME")) != NULL) {
+        asprintf(&path, "%s/.distcc/hosts", home);
+        if (access(path, R_OK) == 0) {
+            ret = dcc_parse_hosts_file(path, ret_list, ret_nhosts);
+            free(path);
+            return ret;
+        } else {
+            rs_trace("not reading %s: %s", path, strerror(errno));
+            free(path);
         }
-        strncpy(*pdst, *psrc, len);
-        (*pdst)[len] = '\0';
-        (*psrc) += len;
     }
 
-    return 0;
+    asprintf(&path, "%s/distcc/hosts", SYSCONFDIR);
+    if (access(path, R_OK) == 0) {
+        ret = dcc_parse_hosts_file(path, ret_list, ret_nhosts);
+        free(path);
+        return ret;
+    } else {
+        rs_trace("not reading %s: %s", path, strerror(errno));
+        free(path);
+    }
+    
+    /* FIXME: Clearer message? */
+    rs_log_warning("no hostlist is set; can't distribute work");
+
+    return EXIT_BAD_HOSTSPEC;
 }
 
+
+
 static int dcc_parse_multiplier(const char **psrc, struct dcc_hostdef *hostdef)
 {
     char *mul;
@@ -186,7 +195,10 @@ static int dcc_parse_ssh_host(struct dcc_hostdef *hostdef,
     if ((ret = dcc_dup_part(&token, &hostdef->user, "@")) != 0)
         return ret;
 
-    assert(token[0] == '@');
+    if (token[0] != '@') {
+        rs_fatal("expected '@' to start ssh token");
+    }
+
     token++;
 
     if ((ret = dcc_dup_part(&token, &hostdef->hostname, "/: \t\n\f")) != 0)
@@ -313,6 +325,12 @@ int dcc_parse_hosts(const char *where,
             return EXIT_OUT_OF_MEMORY;
         }
 
+        /* Store verbatim hostname */
+        if (!(curr->hostdef_string = strndup(token_start, token_len))) {
+            rs_log_crit("failed to allocate hostdef_string");
+            return EXIT_OUT_OF_MEMORY;
+        }
+
         /* Link into list */
         if (prev) {
             prev->next = curr;
@@ -324,8 +342,7 @@ int dcc_parse_hosts(const char *where,
         curr->n_slots = 4;
             
         has_at = (memchr(token_start, '@', token_len) != NULL);
-        /* TODO: Call a separate function to split each type up, then
-         * link the result into the list. */
+
         if (!strncmp(token_start, "localhost", token_len)
             || !strncmp(token_start, "localhost/", strlen("localhost/"))) {
             if ((ret = dcc_parse_localhost(curr, token_start)) != 0)
@@ -354,3 +371,18 @@ int dcc_parse_hosts(const char *where,
     }
 }
 
+
+
+int dcc_free_hostdef(struct dcc_hostdef *host)
+{
+    /* ANSI C requires free() to accept NULL */
+    
+    free(host->user);
+    free(host->hostname);
+    free(host->ssh_command);
+    free(host->hostdef_string);
+    memset(host, 0xf1, sizeof *host);
+    free(host);
+
+    return 0;
+}
