@@ -1,7 +1,6 @@
 /* -*- c-file-style: "java"; indent-tabs-mode: nil; fill-column: 78 -*-
  * 
  * distcc -- A simple distributed compiler system
- * $Header: /data/cvs/distcc/src/io.c,v 1.51 2003/02/24 05:12:37 mbp Exp $ 
  *
  * Copyright (C) 2002, 2003 by Martin Pool <mbp@samba.org>
  *
@@ -60,11 +59,9 @@
 #include <errno.h>
 
 #include <sys/stat.h>
-#ifdef HAVE_SYS_SENDFILE_H
-#  include <sys/sendfile.h>
-#endif /* !HAVE_SYS_SENDFILE_H */
 #include <sys/types.h>
 #include <sys/socket.h>
+#include <sys/select.h>
 
 #include <netinet/in.h>
 #include <netinet/tcp.h>
@@ -76,33 +73,96 @@
 #include "exitcode.h"
 
 
+int dcc_select_for_read(int fd)
+{
+    fd_set fds;
+    FD_ZERO(&fds);
+    FD_SET(fd, &fds);
+    rs_trace("select for read on fd%d", fd);
+    if (select(fd + 1, &fds, NULL, NULL, NULL) == -1) {
+        rs_log_error("select failed: %s", strerror(errno));
+        return EXIT_IO_ERROR;
+    }
+    return 0;
+}
+
+
+int dcc_select_for_write(int fd)
+{
+    fd_set fds;
+    FD_ZERO(&fds);
+    FD_SET(fd, &fds);
+    rs_trace("select for write on fd%d", fd);
+    if (select(fd + 1, NULL, &fds, &fds, NULL) == -1) {
+        rs_log_error("select failed: %s", strerror(errno));
+        return EXIT_IO_ERROR;
+    }
+    return 0;
+}
+
+
 /**
  * Copy @p n bytes from @p ifd to @p ofd.
  *
- * Does not use sendfile(), so @p ifd may be a socket.
+ * Does not use sendfile(), so either one may be a socket.
+ *
+ * In the current code at least one of the files will always be a regular
+ * (disk) file, even though it may not be mmapable.  That should mean that
+ * writes to it will always complete immediately.  That in turn means that on
+ * each pass through the main loop we ought to either completely fill our
+ * buffer, or completely drain it, depending on which one is the disk.
+ *
+ * In future we may put back the ability to feed the compiler from a fifo, in
+ * which case it may be that the writes don't complete.
+ *
+ * We might try selecting on both buffers and handling whichever is ready.
+ * This would require some approximation to a circular buffer though, which
+ * might be more complex.
+ *
+ * However, don't overoptimize this because it has to be replaced to allow for
+ * compression.
  **/
 int dcc_pump_readwrite(int ofd, int ifd, size_t n)
 {
-    char buf[60000], *p;
+    static char buf[262144];    /* we're not recursive */
+    char *p;
     ssize_t r_in, r_out, wanted;
+    int ret;
 
     while (n > 0) {
-         wanted = (n > sizeof buf) ? (sizeof buf) : n;
-         r_in = read(ifd, buf, (size_t) wanted);
-         
-         if (r_in == -1) {
+        wanted = (n > sizeof buf) ? (sizeof buf) : n;
+        r_in = read(ifd, buf, (size_t) wanted);
+
+        if (r_in == -1 && errno == EAGAIN) {
+            if ((ret = dcc_select_for_read(ifd)) != 0)
+                return ret;
+            else
+                continue;
+        } else if (r_in == -1) {
             rs_log_error("failed to read %ld bytes: %s",
                          (long) wanted, strerror(errno));
-            return -1;
-         } else if (r_in == 0) {
-              break;	/* great */
-         }
-         
+            return EXIT_IO_ERROR;
+        } else if (r_in == 0) {
+            rs_log_error("unexpected eof on fd%d", ifd);
+            return EXIT_IO_ERROR;
+        }
+        
         n -= r_in;
         p = buf;
 
+        /* We now have r_in bytes waiting to go out, starting at p.  Keep
+         * going until they're all written out. */
+
         while (r_in > 0) {
             r_out = write(ofd, p, (size_t) r_in);
+            
+            if (r_out == -1 && errno == EAGAIN) {
+                if ((ret = dcc_select_for_write(ofd)) != 0)
+                    return ret;
+                else
+                    continue;
+            }
+            
             if (r_out == -1  ||  r_out == 0) {
                 rs_log_error("failed to write: %s", strerror(errno));
                 return EXIT_IO_ERROR;
@@ -117,35 +177,58 @@ int dcc_pump_readwrite(int ofd, int ifd, size_t n)
 
 
 
-int dcc_readx(int fd,  void *buf, size_t len)
+/**
+ * Read exactly @p len bytes from a file.
+ **/
+int dcc_readx(int fd, void *buf, size_t len)
 {
-	ssize_t r;
-	
-	while (len > 0) {
-		r = read(fd, buf, len);
-		if (r == -1) {
-			rs_log_error("failed to read: %s", strerror(errno));
-                        return EXIT_IO_ERROR;
-		} else if (r == 0) {
-			rs_log_error("unexpected eof on fd%d", fd);
-			return EXIT_TRUNCATED;
-		} else {
-			buf = &((char *) buf)[r];
-			len -= r;
-		}
-	}
+    ssize_t r;
+    int ret;
 
-	return 0;
+    while (len > 0) {
+	r = read(fd, buf, len);
+
+        if (r == -1 && errno == EAGAIN) {
+            if ((ret = dcc_select_for_read(fd)))
+                return ret;
+            else
+                continue;
+        } else if (r == -1) {
+	    rs_log_error("failed to read: %s", strerror(errno));
+	    return EXIT_IO_ERROR;
+	} else if (r == 0) {
+	    rs_log_error("unexpected eof on fd%d", fd);
+	    return EXIT_TRUNCATED;
+	} else {
+	    buf = &((char *) buf)[r];
+	    len -= r;
+	}
+    }
+
+    return 0;
 }
 
 
+/**
+ * Write bytes to an fd.  Keep writing until we're all done or something goes
+ * wrong.
+ *
+ * @returns 0 or exit code.
+ **/
 int dcc_writex(int fd, const void *buf, size_t len)
 {
     ssize_t r;
+    int ret;
 	
     while (len > 0) {
         r = write(fd, buf, len);
-        if (r == -1) {
+
+        if (r == -1 && errno == EAGAIN) {
+            if ((ret = dcc_select_for_write(fd)))
+                return ret;
+            else
+                continue;
+        } else if (r == -1) {
             rs_log_error("failed to write: %s", strerror(errno));
             return EXIT_IO_ERROR;
         } else if (r == 0) {
@@ -161,62 +244,6 @@ int dcc_writex(int fd, const void *buf, size_t len)
 }
 
 
-int dcc_write_str(int fd, const char *what)
-{
-    return dcc_writex(fd, what, strlen(what));
-}
-
-
-int dcc_r_str_alloc(int fd, int l, char **buf)
-{
-     char *s;
-
-     if (l < 0)
-         rs_fatal("oops, l < 0");
-
-/*      rs_trace("read %d byte string", l); */
-
-     s = *buf = malloc((size_t) l + 1);
-     if (!s)
-          rs_log_error("malloc failed");
-     if (dcc_readx(fd, s, (size_t) l))
-          return EXIT_OUT_OF_MEMORY;
-
-     s[l] = 0;
-
-     return 0;
-}
-
-
-int dcc_write_int(int fd, unsigned v)
-{
-	char buf[10];
-	sprintf(buf, "%08x", v);
-	return dcc_writex(fd, buf, 8);
-}
-
-
-
-int dcc_read_int(int fd, unsigned *v)
-{
-    char buf[9], *bum;
-    int ret;    
-	
-    if ((ret = dcc_readx(fd, buf, 8)))
-        return ret;
-    buf[8] = 0;
-    
-    *v = (unsigned) strtoul(buf, &bum, 16);
-    if (bum != &buf[8]) {
-        rs_log_error("failed to parse integer %s", buf);
-        return EXIT_PROTOCOL_ERROR;
-    }
-
-    return 0;
-}
-
-
-
 /**
  * Stick a TCP cork in the socket.  It's not clear that this will help
  * performance, but it might.
@@ -230,9 +257,15 @@ int tcp_cork_sock(int fd, int corked)
         return 0;
     
     if (setsockopt(fd, SOL_TCP, TCP_CORK, &corked, sizeof corked) == -1) {
-        rs_log_notice("setsockopt(corked=%d) failed: %s",
-                      corked, strerror(errno));
-        /* continue anyhow */
+        if (errno == ENOSYS || errno == ENOTSUP) {
+            if (corked)
+                rs_trace("no corks allowed on fd%d", fd);
+            /* no need to complain about not uncorking */
+        } else {
+            rs_log_warning("setsockopt(corked=%d) failed: %s",
+                           corked, strerror(errno));
+            /* continue anyhow */
+        }
     }
 #endif /* def TCP_CORK */
     return 0;
