@@ -43,12 +43,9 @@
  * it.  There shouldn't be many of them, and dead ones will be caught by the
  * tmpreaper.  In any case they're zero bytes.
  *
- * Semaphores might work well here, but the interface is a bit ugly
- * and they have a reputation for being nonportable.
- *
- * @fixme Cygwin (really, Windows) can't handle recursive file locks in the
- * same way as POSIX.  We ought to avoid ever taking the same lock twice from
- * the same process.
+ * Semaphores might work well here, but the interface is a bit ugly and they
+ * are probably not portable to Cygwin.  In particular they can leak if the
+ * process is abruptly terminated, which is likely to happen to distcc.
  */
 
 
@@ -57,7 +54,6 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <unistd.h>
-#include <assert.h>
 #include <string.h>
 #include <fcntl.h>
 #include <errno.h>
@@ -84,15 +80,19 @@ struct dcc_hostdef _dcc_local = {
     0,
     NULL,
     4,                          /* number of tasks */
+    (char *)"localhost",        /* verbatim string */
     NULL    
 };
 
 struct dcc_hostdef *dcc_hostdef_local = &_dcc_local;
 
 
-static char * dcc_make_lock_filename(const char *lockname,
-                                     const struct dcc_hostdef *host,
-                                     int iter)
+/**
+ * Returns a newly allocated buffer.
+ **/
+char * dcc_make_lock_filename(const char *lockname,
+                              const struct dcc_hostdef *host,
+                              int iter)
 {
     char * buf;
     const char *tempdir;
@@ -161,7 +161,7 @@ int dcc_unlock(int lock_fd)
 /**
  * Open a lockfile, creating if it does not exist.
  **/
-static int dcc_open_lockfile(const char *fname, int *plockfd)
+int dcc_open_lockfile(const char *fname, int *plockfd)
 {
     /* Create if it doesn't exist.  We don't actually do anything with
      * the file except lock it.*/
@@ -204,8 +204,7 @@ int dcc_lock_host(const char *lockname,
     }        
 
     if (sys_lock(*lock_fd, block) == 0) {
-        /* TODO: Print better readable form of hostdef. */
-        rs_trace("got %s lock on %s slot %d", lockname, host->hostname, slot);
+        rs_trace("got %s lock on %s slot %d", lockname, host->hostdef_string, slot);
         free(fname);
         return 0;
     } else {
@@ -214,7 +213,7 @@ int dcc_lock_host(const char *lockname,
         case EWOULDBLOCK:
 #endif
         case EAGAIN:
-        case EACCES: /* HP-UX gives this for exclusion */
+        case EACCES: /* HP-UX and Cygwin give this for exclusion */
             rs_trace("%s is busy", fname);
             ret = EXIT_BUSY;
             break;
