@@ -25,3 +25,11 @@ int dcc_lock_host(const char *lockname,
                   int *lock_fd);
 
 int dcc_unlock(int lock_fd);
+
+char * dcc_make_lock_filename(const char *lockname,
+                              const struct dcc_hostdef *host,
+                              int iter);
+
+int dcc_open_lockfile(const char *fname, int *plockfd);
+
+
