@@ -30,11 +30,9 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <unistd.h>
-#include <assert.h>
 #include <string.h>
 #include <fcntl.h>
 #include <errno.h>
-#include <assert.h>
 
 #include <sys/stat.h>
 #ifdef HAVE_SYS_SENDFILE_H
@@ -120,8 +118,8 @@ static ssize_t sys_sendfile(int ofd, int ifd, off_t *offset, size_t size)
         *offset += ret;
         return ret;
     } else {
-        rs_log_error("don't know how to handle return %d from HP-UX sendfile",
-                     ret);
+        rs_log_error("don't know how to handle return %ld from HP-UX sendfile",
+                     (long) ret);
         return -1;
     }
 }
@@ -150,6 +148,7 @@ int dcc_pump_sendfile(int ofd, int ifd, size_t size)
 {
     ssize_t sent;
     off_t offset = 0;
+    int ret;
 
     while (size) {
         /* Handle possibility of partial transmission, e.g. if
@@ -169,18 +168,22 @@ int dcc_pump_sendfile(int ofd, int ifd, size_t size)
 
                 rs_log_info("decided to use read/write rather than sendfile");
                 return dcc_pump_readwrite(ofd, ifd, size);
+            } else if (errno == EAGAIN) {
+                if ((ret = dcc_select_for_write(ofd)) != 0)
+                    return ret;
+                rs_trace("select() returned, continuing to write");
             } else {
                 rs_log_error("sendfile failed: %s", strerror(errno));
-                return -1;
+                return EXIT_IO_ERROR;
             }
         } else if (sent == 0) {
             rs_log_error("sendfile returned 0? can't cope");
-            return -1;
+            return EXIT_IO_ERROR;
         } else if (sent != (ssize_t) size) {
             /* offset is automatically updated by sendfile. */
             size -= sent;
-            rs_log_notice("sendfile: partial transmission of %d bytes; retrying %ld @%ld",
-                          sent, (long) size, (long) offset);
+            rs_log_notice("sendfile: partial transmission of %ld bytes; retrying %ld @%ld",
+                          (long) sent, (long) size, (long) offset);
         } else {
             /* normal case, everything was sent. */
             break;
