@@ -57,7 +57,6 @@
 #include <stdlib.h>
 #include <setjmp.h>
 #include <unistd.h>
-#include <assert.h>
 #include <string.h>
 #include <fcntl.h>
 #include <errno.h>
@@ -76,26 +75,28 @@
 #include "util.h"
 #include "rpc.h"
 #include "exitcode.h"
-#include "opt.h"
+#include "dopt.h"
 #include "tempfile.h"
 #include "bulk.h"
 #include "exec.h"
 #include "srvnet.h"
 #include "filename.h"
+#include "hosts.h"
 
 
 static int dcc_r_request_header(int ifd)
 {
     int vers;
+    int ret;
     
-    if (dcc_r_token_int(ifd, "DIST", &vers) == -1) {
+    if ((ret = dcc_r_token_int(ifd, "DIST", &vers)) != 0) {
         rs_log_error("client did not provide distcc magic fairy dust");
-        return -1;
+        return ret;
     }
 
     if (vers != PROTO_VER) {
         rs_log_error("client version is %d, i am %d", vers, 1);
-        return -1;
+        return EXIT_PROTOCOL_ERROR;
     }
     
     return 0;
@@ -110,11 +111,12 @@ static int dcc_r_argv(int ifd, /*@out@*/ char ***argv)
     int i;
     int argc;
     char **a;
+    int ret;
 
     *argv = NULL;
      
     if (dcc_r_token_int(ifd, "ARGC", &argc))
-        return -1;
+        return EXIT_PROTOCOL_ERROR;
 
     rs_trace("reading %d arguments from job submission", argc);
     
@@ -123,54 +125,29 @@ static int dcc_r_argv(int ifd, /*@out@*/ char ***argv)
     *argv = a = (char **) calloc((size_t) argc+1, sizeof a[0]);
     if (a == NULL) {
         rs_log_error("alloc failed");
-        return -1;
+        return EXIT_OUT_OF_MEMORY;
     }
     a[argc] = NULL;
 
     for (i = 0; i < argc; i++) {
         int a_len;
         
-        if (dcc_r_token_int(ifd, "ARGV", &a_len))
-            return -1;
+        if ((ret = dcc_r_token_int(ifd, "ARGV", &a_len)))
+            return ret;
 
-        if (dcc_r_str_alloc(ifd, a_len, &a[i]))
-            return -1;
+        if ((ret = dcc_r_str_alloc(ifd, a_len, &a[i])))
+            return ret;
         
         rs_trace("argv[%d] = \"%s\"", i, a[i]);
     }
-    
-    {
-        char *astr;
-        astr = dcc_argv_tostr(a);
-        rs_log_info("got arguments: %s", astr);
-        free(astr);
-    }
+
+    dcc_trace_argv("got arguments", a);
     
     return 0;
 }
 
 
 
-#if 0
-/* Not used at the moment because the server never kills off
-   the compiler, but it might be in the future */
-
-static int dcc_kill_compiler(pid_t pid, int signum)
-{
-    rs_log_info("killing compiler pid %d with signal %d",
-                (int) pid, signum);
-
-    if (kill(pid, signum)) {
-        rs_log_warning("kill(%d, %d) failed: %s", (int) pid, signum,
-                       strerror(errno));
-        return -1;
-    }
-    
-    return 0;
-}
-#endif
-
-
 /**
  * Copy all server messages to the error file, so that they can be
  * echoed back to the client if necessary.
@@ -202,42 +179,6 @@ static void dcc_terminate_group(int whichsig)
     kill(0, whichsig);
 }
 
-/**
- * Actually run the compiler on the server.
- *
- * This whole function is run with a signal handler set such that if a
- * child terminates, it will immediately interrupt system calls and
- * return.
- *
- * This is called with SIGCHLD on, so any of the calls here may fail
- * with EINTR if the child exits.  I think that's OK, though, because
- * if that happens then the compiler has unexpectedly exited.
- *
- * @param argv     Compiler command line, already modified for temporary files.
- *
- * @param netfd    fd of client connection.
- **/
-static int dcc_server_compile(char **argv, int netfd,
-                              const char *temp_i,
-                              const char *temp_out, const char *temp_err)
-{
-    int i_size;
-    int ret;
-    pid_t pid;
-
-    pid = 0;
-
-    if ((ret = dcc_r_token_int(netfd, "DOTI", &i_size)))
-        return ret;
-
-    rs_log_info("input file is %d bytes", i_size);
-
-    /* read input before spawning */
-    if ((ret = dcc_r_file(netfd, temp_i, i_size)) != 0)
-        return ret;
-
-    return dcc_spawn_child(argv, &pid, "/dev/null", temp_out, temp_err);
-}
 
 
 /**
@@ -298,6 +239,8 @@ int dcc_accept_job(int netfd)
      * mode. */
     dcc_add_log_to_file(err_fname);
 
+    /* TODO: Don't call this for ssh connections */
+    
     /* Log client name and check access if appropriate. */
     if ((ret = dcc_check_client(netfd)) != 0)
         return ret;
@@ -359,7 +302,7 @@ int dcc_accept_job(int netfd)
     }
 
     dcc_report_rusage(argv[0], u_us, s_us);
-    dcc_critique_status(status, argv[0], dcc_gethostname());
+    dcc_critique_status(status, argv[0], dcc_hostdef_local, 0);
     tcp_cork_sock(netfd, 0);
 
     rs_log_info("complete; output file: %ld bytes", (long) o_size);
