@@ -21,7 +21,7 @@
  */
 
 /* srvnet.c */
-int open_socket_in(int port);
+int dcc_socket_listen(int port, int *fd, const char *listen_addr);
 int is_a_socket(int fd);
 int dcc_check_client(int fd);
 
