@@ -1,9 +1,8 @@
 /* -*- c-file-style: "java"; indent-tabs-mode: nil -*-
  * 
  * distcc -- A simple distributed compiler system
- * $Header: /data/cvs/distcc/src/tempfile.c,v 1.26 2003/02/25 00:27:25 mbp Exp $ 
  *
- * Copyright (C) 2002 by Martin Pool <mbp@samba.org>
+ * Copyright (C) 2002, 2003 by Martin Pool <mbp@samba.org>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -35,7 +34,6 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <unistd.h>
-#include <assert.h>
 #include <string.h>
 #include <fcntl.h>
 #include <errno.h>
@@ -47,6 +45,10 @@
 #include "trace.h"
 #include "util.h"
 #include "tempfile.h"
+#include "snprintf.h"
+#include "exitcode.h"
+
+
 
 /**
  * @file
@@ -104,7 +106,7 @@ static const char *dcc_get_tmp_top(void)
  *
  * @param mode Permission bits for new fifo.
  *
- * @returns 0 for OK, or -1 for error.
+ * @returns 0 for OK, or error.
  **/
 int dcc_mkfifo(const char *fifo_name)
 {
@@ -113,12 +115,12 @@ int dcc_mkfifo(const char *fifo_name)
      * previous invocation with the same pid. */
     
     if (dcc_remove_if_exists(fifo_name))
-        return -1;
+        return EXIT_IO_ERROR;
 
     if (mkfifo(fifo_name, S_IRUSR|S_IWUSR) == -1) {
         rs_log_warning("failed to make fifo %s: %s", fifo_name,
                        strerror(errno));
-        return -1;
+        return EXIT_IO_ERROR;
     }
 
     return 0;
@@ -140,7 +142,7 @@ int dcc_setup_tempdir(void)
     /* size includes trailing nul */
     need_len = strlen(tmp) + strlen("/distcc_") + 8 + 1;
     tempdir = malloc(need_len);
-    if (snprintf(tempdir, need_len, "%s/distcc_%08x", tmp, getuid())
+    if (snprintf(tempdir, need_len, "%s/distcc_%08x", tmp, (int) getuid())
         != need_len - 1) {
         rs_fatal("tempdir too small??");
     }
@@ -154,20 +156,20 @@ int dcc_setup_tempdir(void)
          * directory. */
         if (lstat(tempdir, &buf) == -1) {
             rs_log_error("lstat %s failed: %s", tempdir, strerror(errno));
-            return -1;            
+            return EXIT_IO_ERROR;
         } else if (!S_ISDIR(buf.st_mode)) {
             rs_log_error("%s is not a directory", tempdir);
-            return -1;
+            return EXIT_IO_ERROR;
         } else if (buf.st_mode & (S_IWGRP | S_IWOTH)) {
             rs_log_error("permissions on %s (%#o) seem insecure", tempdir,
-                         buf.st_mode);
-            return -1;
+                         (int) buf.st_mode);
+            return EXIT_IO_ERROR;
         } else {
             return 0;
         }
     } else {
         rs_log_error("mkdir %s failed: %s", tempdir, strerror(errno));
-        return -1;
+        return EXIT_IO_ERROR;
     }
 }
 
@@ -184,7 +186,9 @@ char *dcc_make_tmpnam(const char *prefix, const char *suffix)
     for (i = 0; cleanups[i]; i++)
         ;
 
-    assert(i < N_CLEANUPS);
+    if (i >= N_CLEANUPS) {
+        rs_fatal("too many cleanups");
+    }
 
     /* NOTE: Make sure this lines up with the printf statement.  C sucks. */
     need_len = strlen(tempdir) + 1 + strlen(prefix) + 1 + 10 
