@@ -39,13 +39,8 @@ Copyright (C) 2002, 2003 by Martin Pool
 # abstract superclasses just provide methods that can be called,
 # rather than establishing default behaviour.
 
-# TODO: If the server fails with error 102, then try a different port.
-
 # TODO: Run the server in a different directory from the clients
 
-# TODO: Try compiling a file locally and remotely; check that they are
-# the same.
-
 # TODO: Compile locally to .s, and assemble remotely; check that
 # works.
 
@@ -74,10 +69,6 @@ Copyright (C) 2002, 2003 by Martin Pool
 # TODO: Argument scanning tests should be run with various hostspecs,
 # because that makes a big difference to how the client handles them.
 
-# TODO: Test --no-fork mode
-
-# TODO: Test --inetd mode by creating a socketpair ourselves
-
 # TODO: Test that ccache gets hits when calling distcc.  Presumably
 # this is skipped if we can't find ccache.  Need to parse `ccache -s`.
 
@@ -140,7 +131,8 @@ Copyright (C) 2002, 2003 by Martin Pool
 # TODO: Test scheduler.  Perhaps run really slow jobs to make things
 # deterministic, and test that they're dispatched in a reasonable way.
 
-# TODO: Test generating dependencies with -MD.
+# TODO: Test generating dependencies with -MD.  Possibly can't be
+# done.
 
 # TODO: Test a nasty cpp that always writes to stdout regardless of
 # -o.
@@ -150,11 +142,16 @@ Copyright (C) 2002, 2003 by Martin Pool
 
 # TODO: Test that recursion safeguard works.
 
-# TODO: Test using h_parsemask
+# TODO: Test masquerade mode.  Requires us to create symlinks in a
+# special directory on the path.
+
+# TODO: Test SSH mode.  May need to skip if we can't ssh to this
+# machine.  Perhaps provide a little null-ssh.
+
+# TODO: Test path stripping.
+
+# TODO: Test backoff from downed hosts.
 
-# TODO: Check using daemon --allow option for both allowed and denied
-# cases.  Check with multiple addresses.  Perhaps count only on
-# 127.0.0.1 - this ought to be enough.
 
 import time, sys, string, os, types, re, popen2, pprint
 import signal, os.path, string
@@ -170,29 +167,13 @@ EXIT_COMPILER_CRASHED        = 104
 EXIT_OUT_OF_MEMORY           = 105
 EXIT_BAD_HOSTSPEC            = 106
 EXIT_COMPILER_MISSING        = 110
-
+EXIT_ACCESS_DENIED           = 113
 
 class SimpleDistCC_Case(comfychair.TestCase):
     '''Abstract base class for distcc tests'''
-    def __init__(self):
-        self.cmd_log = ''
-
-    def setUp(self):
-        self.enterRundir()
+    def setup(self):
         self.stripEnvironment()
 
-    def enterRundir(self):
-        """Create and enter a temporary directory for this test."""
-        self.basedir = os.getcwd()
-        self.rundir = os.path.join(self.basedir,
-                                   'testtmp', 
-                                   self.__class__.__name__)
-        self.tmpdir = os.path.join(self.rundir, 'tmp')
-        os.system("rm -fr %s" % self.rundir)
-        os.makedirs(self.tmpdir)
-        os.system("mkdir -p %s" % self.rundir)
-        os.chdir(self.rundir)
-
     def stripEnvironment(self):
         """Remove all DISTCC variables from the environment, so that
         the test is not affected by the development environment."""
@@ -204,36 +185,6 @@ class SimpleDistCC_Case(comfychair.TestCase):
                 del os.environ[key]
         os.environ['TMPDIR'] = self.tmpdir
 
-    def tearDown(self):
-        self.leaveRundir()
-
-    def leaveRundir(self):
-        os.chdir(self.basedir)
-
-    def explainFailure(self, exc_info):
-        import traceback
-        print "-----------------------------------------------------------------"
-        traceback.print_exc(file=sys.stdout)
-        print self.cmd_log
-        print "-----------------------------------------------------------------"
-    
-    def fail(self, msg):
-        raise AssertionError(msg)
-
-    def assertEquals(self, a, b):
-        if not a == b:
-            raise AssertionError("assertEquals failed: %s" % `(a, b)`)
-            
-    def assertNotEqual(self, a, b):
-        if a == b:
-            raise AssertionError("assertNotEqual failed: %s" % `(a, b)`)
-
-    def assertReMatch(self, needle, haystack):
-        """Look for the regular expression re in haystack"""
-        if not re.search(needle, haystack):
-            self.fail("couldn't find re %s in %s" % (`needle`, `haystack`))
-            
-
 
 class WithDaemon_Case(SimpleDistCC_Case):
     """Start the daemon, and then run a command locally against it.
@@ -241,11 +192,12 @@ class WithDaemon_Case(SimpleDistCC_Case):
 The daemon doesn't detach until it has bound the network interface, so
 as soon as that happens we can go ahead and start the client."""
 
-    def setUp(self):
-        SimpleDistCC_Case.setUp(self)
+    def setup(self):
+        import random
+        SimpleDistCC_Case.setup(self)
         self.daemon_pidfile = os.path.join(os.getcwd(), "daemonpid.tmp")
         self.daemon_logfile = os.path.join(os.getcwd(), "distccd.log")
-        self.server_port = 42000
+        self.server_port = random.randint(42000, 43000)
         self.startDaemon()
         self.setupEnv()
 
@@ -255,14 +207,17 @@ as soon as that happens we can go ahead and start the client."""
         os.environ['DISTCC_VERBOSE'] = '1'
 
 
-    def tearDown(self):
-        self.killDaemon()
-        SimpleDistCC_Case.tearDown(self)
+    def teardown(self):
+        SimpleDistCC_Case.teardown(self)
 
 
     def killDaemon(self):
         import signal, time
-        pid = int(open(self.daemon_pidfile, 'rt').read())
+        try:
+            pid = int(open(self.daemon_pidfile, 'rt').read())
+        except IOError:
+            # the daemon probably already exited, perhaps because of a timeout
+            return
         os.kill(pid, signal.SIGTERM)
 
         # We can't wait on it, because it detached.  So just keep
@@ -275,6 +230,18 @@ as soon as that happens we can go ahead and start the client."""
             time.sleep(0.2)
 
 
+    def daemon_command(self):
+        """Return command to start the daemon"""
+        return ("distccd --verbose --lifetime=%d --daemon --log-file %s "
+                "--pid-file %s --port %d --allow 127.0.0.1"
+                % (self.daemon_lifetime(),
+                   self.daemon_logfile, self.daemon_pidfile, self.server_port))
+
+    def daemon_lifetime(self):
+        # Enough for most tests, even on a fairly loaded machine.
+        # Might need more for long-running tests.
+        return 60
+
     def startDaemon(self):
         """Start a daemon in the background, return its pid"""
         # The daemon detaches once it has successfully bound the
@@ -282,15 +249,15 @@ as soon as that happens we can go ahead and start the client."""
         # find out straight away.  If it starts successfully, then we
         # can go ahead and try to connect.
         
-        cmd = ("distccd --verbose --daemon --log-file %s --pid-file %s --port %d"
-               % (self.daemon_logfile, self.daemon_pidfile, self.server_port))
-        self.runCmd(cmd)
+        cmd = self.daemon_command()
+        self.runcmd(cmd)
+        self.add_cleanup(self.killDaemon)
 
 
 
 
 class StartStopDaemon_Case(WithDaemon_Case):
-    def runTest(self):
+    def runtest(self):
         pass
 
 
@@ -299,37 +266,38 @@ class VersionOption_Case(SimpleDistCC_Case):
 
     This is also a good test that the programs were built properly and are
     executable."""
-    def runTest(self):
+    def runtest(self):
+        import string
         for prog in 'distcc', 'distccd':
-            out = self.runCmd("%s --version" % prog)
+            out, err = self.runcmd("%s --version" % prog)
             assert out[-1] == '\n'
             out = out[:-1]
-            line1,trash = out.split('\n')
+            line1,trash = string.split(out, '\n')
             self.assert_re_match(r'^%s [\w.-]+ [.\w-]+ \(protocol 1\) \(default port 3632\)$'
                                  % prog, line1)
 
 
 class HelpOption_Case(SimpleDistCC_Case):
     """Test --help is reasonable."""
-    def runTest(self):
+    def runtest(self):
         for prog in 'distcc', 'distccd':
-            out = self.runCmd(prog + " --help")
-            self.assert_regexp("Usage:", out)
+            out, err = self.runcmd(prog + " --help")
+            self.assert_re_search("Usage:", out)
 
 
 class BogusOption_Case(SimpleDistCC_Case):
     """Test handling of --bogus-option.
 
     Now that we support implicit compilers, this is passed to gcc, which returns 1."""
-    def runTest(self):
-        self.runCmd("distcc --bogus-option", 1)
-        self.runCmd("distccd --bogus-option", EXIT_BAD_ARGUMENTS)
+    def runtest(self):
+        self.runcmd("distcc --bogus-option", 1)
+        self.runcmd("distccd --bogus-option", EXIT_BAD_ARGUMENTS)
 
 
 class GccOptionsPassed_Case(SimpleDistCC_Case):
     """Test that options following the compiler name are passed to the compiler."""
-    def runTest(self):
-        out = self.runCmd("DISTCC_HOSTS=localhost distcc gcc --help")
+    def runtest(self):
+        out, err = self.runcmd("DISTCC_HOSTS=localhost distcc gcc --help")
         if re.search('distcc', out):
             raise ("gcc help contains \"distcc\": \"%s\"" % out)
         self.assert_re_match(r"^Usage: gcc", out)
@@ -337,7 +305,7 @@ class GccOptionsPassed_Case(SimpleDistCC_Case):
 
 class StripArgs_Case(SimpleDistCC_Case):
     """Test -D and -I arguments are removed"""
-    def runTest(self):
+    def runtest(self):
         cases = (("gcc -c hello.c", "gcc -c hello.c"),
                  ("cc -Dhello hello.c -c", "cc hello.c -c"),
                  ("gcc -g -O2 -W -Wall -Wshadow -Wpointer-arith -Wcast-align -c -o h_strip.o h_strip.c",
@@ -357,13 +325,13 @@ class StripArgs_Case(SimpleDistCC_Case):
                   "cc -o nsinstall.o -c -fPIC -Wall -W -Wno-unused -Wpointer-arith -Wcast-align -pedantic -Wno-long-long -pthread -pipe -g nsinstall.c"),
                  )
         for cmd, expect in cases:
-            o = self.runCmd("h_strip %s" % cmd)
+            o, err = self.runcmd("h_strip %s" % cmd)
             if o[-1] == '\n': o = o[:-1]
-            self.assertEquals(o, expect)
+            self.assert_equal(o, expect)
 
 
 class IsSource_Case(SimpleDistCC_Case):
-    def runTest(self):
+    def runtest(self):
         """Test distcc's method for working out whether a file is source"""
         cases = (( "hello.c",          "source",       "not-preprocessed" ),
                  ( "hello.cpp",        "source",       "not-preprocessed" ),
@@ -375,7 +343,7 @@ class IsSource_Case(SimpleDistCC_Case):
                  ( "boot.s",           "not-source",   "not-preprocessed" ),
                  ( "boot.S",           "not-source",   "not-preprocessed" ))
         for f, issrc, iscpp in cases:
-            o = self.runCmd("h_issource '%s'" % f)
+            o, err = self.runcmd("h_issource '%s'" % f)
             expected = ("%s %s\n" % (issrc, iscpp))
             if o != expected:
                 raise AssertionError("issource %s gave %s, expected %s" %
@@ -385,7 +353,7 @@ class IsSource_Case(SimpleDistCC_Case):
 
 class ScanArgs_Case(SimpleDistCC_Case):
     '''Test understanding of gcc command lines'''
-    def runTest(self):
+    def runtest(self):
         cases = [("gcc -c hello.c", "distribute", "hello.c", "hello.o"),
                  ("gcc hello.c", "local"),
                  ("gcc -o /tmp/hello.o -c ../src/hello.c", "distribute", "../src/hello.c", "/tmp/hello.o"),
@@ -413,10 +381,10 @@ class ScanArgs_Case(SimpleDistCC_Case):
                  ("-S -c -o - foo.c", "local"),
                  ]
         for tup in cases:
-            self.checkScanArgs(*tup)
+            apply(self.checkScanArgs, tup)
 
     def checkScanArgs(self, ccmd, mode, input=None, output=None):
-        o = self.runCmd("h_scanargs %s" % ccmd)
+        o, err = self.runcmd("h_scanargs %s" % ccmd)
         o = o[:-1]                      # trim \n
         os = string.split(o)
         if mode != os[0]:
@@ -434,7 +402,7 @@ class ScanArgs_Case(SimpleDistCC_Case):
 
 class ImplicitCompilerScan_Case(ScanArgs_Case):
     '''Test understanding of commands with no compiler'''
-    def runTest(self):
+    def runtest(self):
         cases = [("-c hello.c",            "distribute", "hello.c", "hello.o"),
                  ("hello.c -c",            "distribute", "hello.c", "hello.o"),
                  ("-o hello.o -c hello.c", "distribute", "hello.c", "hello.o"),
@@ -446,37 +414,37 @@ class ImplicitCompilerScan_Case(ScanArgs_Case):
             
 
 class ExtractExtension_Case(SimpleDistCC_Case):
-    def runTest(self):
+    def runtest(self):
         """Test extracting extensions from filenames"""
         for f, e in (("hello.c", ".c"),
                      ("hello.cpp", ".cpp"),
                      ("hello.2.4.4.4.c", ".c"),
                      (".foo", ".foo"),
                      ("gcc", "(NULL)")):
-            out = self.runCmd("h_exten '%s'" % f)
+            out, err = self.runcmd("h_exten '%s'" % f)
             assert out == e
 
 
 class DaemonBadPort_Case(SimpleDistCC_Case):
-    def runTest(self):
+    def runtest(self):
         """Test daemon invoked with invalid port number"""
-        self.runCmd("distccd --log-file=distccd.log --port 80000",
-                    EXIT_BIND_FAILED)
+        self.runcmd("distccd --log-file=distccd.log --lifetime=10 --port 80000",
+                    EXIT_BAD_ARGUMENTS)
         self.assert_no_file("daemonpid.tmp")
 
 
 class InvalidHostSpec_Case(SimpleDistCC_Case):
-    def runTest(self):
+    def runtest(self):
         """Test various invalid DISTCC_HOSTS
         
         See also test_parse_host_spec, which tests valid specifications."""
         for spec in ["", "    ", "\t", "  @ ", ":", "mbp@", "angry::", ":4200"]:
-            self.runCmd("DISTCC_HOSTS=\"%s\" h_hosts -v" % spec,
+            self.runcmd("DISTCC_HOSTS=\"%s\" h_hosts -v" % spec,
                          EXIT_BAD_HOSTSPEC)
 
 
 class ParseHostSpec_Case(SimpleDistCC_Case):
-    def runTest(self):
+    def runtest(self):
         """Check operation of dcc_parse_hosts_env.
 
         Passes complex environment variables to h_hosts, which is a C wrapper
@@ -494,18 +462,19 @@ class ParseHostSpec_Case(SimpleDistCC_Case):
    4 TCP angry 4204
    4 TCP ipv4-localhost 3632
 """
-        out = self.runCmd("DISTCC_HOSTS=\"%s\" h_hosts" % spec)
+        out, err = self.runcmd("DISTCC_HOSTS=\"%s\" h_hosts" % spec)
         assert out == expected, "expected %s\ngot %s" % (`expected`, `out`)
 
 
 class Compilation_Case(WithDaemon_Case):
     '''Test distcc by actually compiling a file'''
-    def setUp(self):
-        WithDaemon_Case.setUp(self)
+    def setup(self):
+        WithDaemon_Case.setup(self)
         self.createSource()
 
-    def runTest(self):
+    def runtest(self):
         self.compile()
+        self.link()
         self.checkBuiltProgram()
 
     def createSource(self):
@@ -518,13 +487,24 @@ class Compilation_Case(WithDaemon_Case):
         return "testtmp.c"              # default
     
     def compile(self):
-        msgs = (self.runCmd(self.compileCmd()) +
-                self.runCmd(self.linkCmd()))
-        self.checkCompileMsgs(msgs)
+        cmd = self.compileCmd()
+        out, err = self.runcmd(cmd)
+        if out != '':
+            self.fail("compiler command %s produced output:\n%s" % (`cmd`, out))
+        if err != '':
+            self.fail("compiler command %s produced error:\n%s" % (`cmd`, err))
+
+    def link(self):
+        cmd = self.linkCmd()
+        out, err = self.runcmd(cmd)
+        if out != '':
+            self.fail("command %s produced output:\n%s" % (`cmd`, `out`))
+        if err != '':
+            self.fail("command %s produced error:\n%s" % (`cmd`, `err`))
 
     def compileCmd(self):
         """Return command to compile source and run tests"""
-        return "distcc cc -o testtmp.o -c %s" % (self.sourceFilename())
+        return "DISTCC_FALLBACK=0 distcc cc -o testtmp.o -c %s" % (self.sourceFilename())
 
     def linkCmd(self):
         return "distcc cc -o testtmp testtmp.o"
@@ -536,8 +516,9 @@ class Compilation_Case(WithDaemon_Case):
 
     def checkBuiltProgram(self):
         '''Check compile results.  By default, just try to execute.'''
-        msgs = self.runCmd("./testtmp")
+        msgs, errs = self.runcmd("./testtmp")
         self.checkBuiltProgramMsgs(msgs)
+        self.assert_equal(errs, '')
 
     def checkBuiltProgramMsgs(self, msgs):
         pass
@@ -556,15 +537,15 @@ int main(void) {
 """
 
     def checkBuiltProgramMsgs(self, msgs):
-        self.assertEquals(msgs, "hello world\n")
+        self.assert_equal(msgs, "hello world\n")
 
 
 
 
 class MultipleCompile_Case(Compilation_Case):
     """Test compiling several files from one line"""
-    def setUp(self):
-        WithDaemon_Case.setUp(self)
+    def setup(self):
+        WithDaemon_Case.setup(self)
         open("test1.c", "w").write("const char *msg = \"hello foreigner\";")
         open("test2.c", "w").write("""#include <stdio.h>
 
@@ -575,9 +556,9 @@ int main(void) {
 }
 """)
         
-    def runTest(self):
-        self.runCmd("distcc cc -c test1.c test2.c")
-        self.runCmd("distcc cc -o test test1.o test2.o")
+    def runtest(self):
+        self.runcmd("distcc cc -c test1.c test2.c")
+        self.runcmd("distcc cc -o test test1.o test2.o")
         
 
 
@@ -586,10 +567,11 @@ class CppError_Case(CompileHello_Case):
     def source(self):
         return '#error "not tonight dear"\n'
 
-    def runTest(self):
+    def runtest(self):
         cmd = "distcc -c testtmp.c"
-        msgs = self.runCmd(cmd, expectedResult=1)
-        self.assert_regexp("not tonight dear", msgs)
+        msgs, errs = self.runcmd(cmd, expectedResult=1)
+        self.assert_re_search("not tonight dear", errs)
+        self.assert_equal(msgs, '')
     
 
 class BadInclude_Case(Compilation_Case):
@@ -598,14 +580,13 @@ class BadInclude_Case(Compilation_Case):
         return """#include <nosuchfilehere.h>
 """
 
-    def runTest(self):
-        self.runCmd("distcc cc -o testtmp.o -c testtmp.c", 1)
+    def runtest(self):
+        self.runcmd("distcc cc -o testtmp.o -c testtmp.c", 1)
 
 
 class PreprocessPlainText_Case(Compilation_Case):
     """Try using cpp on something that's not C at all"""
-    def setUp(self):
-        self.enterRundir()
+    def setup(self):
         self.stripEnvironment()
         self.createSource()
 
@@ -619,15 +600,15 @@ large foo!
 /* comment ca? */
 """
 
-    def runTest(self):
+    def runtest(self):
         # -P means not to emit linemarkers
-        self.runCmd("distcc gcc -E testtmp.c -o testtmp.out")
+        self.runcmd("distcc gcc -E testtmp.c -o testtmp.out")
         out = open("testtmp.out").read()
         # It's a bit hard to know the exact value, because different versions of
         # GNU cpp seem to handle the whitespace differently.
-        self.assert_regexp("small foo!", out)
+        self.assert_re_search("small foo!", out)
 
-    def tearDown(self):
+    def teardown(self):
         # no daemon is run for this test
         pass
         
@@ -635,18 +616,18 @@ large foo!
 class NoDetachDaemon_Case(CompileHello_Case):
     """Check that without --no-detach the server goes into the background and can be stopped."""
     def startDaemon(self):
-        cmd = ["distccd",  "--no-detach", "--verbose",
-               "--log-file", self.daemon_logfile, "--pid-file", self.daemon_pidfile,
-               "--port", str(self.server_port)]
-        self.pid = self.runCmdNoWait(cmd)
+        cmd = ("distccd --no-detach --daemon --verbose --log-file %s --pid-file %s --port %d --allow 127.0.0.1" %
+               (self.daemon_logfile, self.daemon_pidfile, self.server_port))
+        self.pid = self.runcmd_background(cmd)
+        self.add_cleanup(self.killDaemon)
         time.sleep(.5)             # give it a bit of time to bind port
 
     def killDaemon(self):
         import signal
         os.kill(self.pid, signal.SIGTERM)
         pid, ret = os.wait()
-        self.assertEquals(self.pid, pid)
-        self.assertEquals(ret, 0)
+        self.assert_equal(self.pid, pid)
+        self.assert_equal(ret, 0)
 
         
 
@@ -678,24 +659,30 @@ int main(void) {
         return "distcc cc -c -o testtmp.o '-DMESSAGE=\"hello world\"' testtmp.c"
 
     def checkBuiltProgramMsgs(self, msgs):
-        self.assertEquals(msgs, "hello world\n")
+        self.assert_equal(msgs, "hello world\n")
 
 
 class ThousandFold_Case(CompileHello_Case):
     """Try repeated simple compilations"""
-    def runTest(self):
+    def daemon_lifetime(self):
+        return 120
+    
+    def runtest(self):
         # may take about a minute or so
         for i in xrange(1000):
-            self.runCmd("distcc cc -o testtmp.o -c testtmp.c")
+            self.runcmd("distcc cc -o testtmp.o -c testtmp.c")
 
 
 class Concurrent_Case(CompileHello_Case):
     """Try many compilations at the same time"""
-    def runTest(self):
+    def daemon_lifetime(self):
+        return 120
+    
+    def runtest(self):
         # may take about a minute or so
         pids = {}
         for i in xrange(100):
-            kid = self.runCmdNoWait("distcc cc -o testtmp.o -c testtmp.c".split())
+            kid = self.runcmd_background("distcc cc -o testtmp.o -c testtmp.c")
             pids[kid] = kid
         while len(pids):
             pid, status = os.wait()
@@ -722,9 +709,13 @@ class BigAssFile_Case(Compilation_Case):
             f.write("int i%06d = %d;\n" % (i, i))
         f.close()
 
-    def runTest(self):
-        self.runCmd("distcc cc -c %s" % "testtmp.c")
-        self.runCmd("distcc cc -o testtmp testtmp.o")
+    def runtest(self):
+        self.runcmd("distcc cc -c %s" % "testtmp.c")
+        self.runcmd("distcc cc -o testtmp testtmp.o")
+
+
+    def daemon_lifetime(self):
+        return 300
 
 
 
@@ -735,7 +726,7 @@ class BinFalse_Case(Compilation_Case):
     because it has to cope with the open() on the fifo being
     interrupted.
 
-    distcc doesn't know that /bin/false is not a compiler, but it does
+    distcc doesn't know that 'false' is not a compiler, but it does
     need a command line that looks like a compiler invocation.
 
     We have to use a .i file so that distcc does not try to preprocess it.
@@ -743,8 +734,8 @@ class BinFalse_Case(Compilation_Case):
     def createSource(self):
         open("testtmp.i", "wt").write("int main() {}")
         
-    def runTest(self):
-        self.runCmd("distcc /bin/false -c testtmp.i", 1)
+    def runtest(self):
+        self.runcmd("distcc false -c testtmp.i", 1)
 
 
 class BinTrue_Case(Compilation_Case):
@@ -754,7 +745,7 @@ class BinTrue_Case(Compilation_Case):
     because it has to cope with the open() on the fifo being
     interrupted.
 
-    distcc doesn't know that /bin/false is not a compiler, but it does
+    distcc doesn't know that 'true' is not a compiler, but it does
     need a command line that looks like a compiler invocation.
 
     We have to use a .i file so that distcc does not try to preprocess it.
@@ -762,8 +753,8 @@ class BinTrue_Case(Compilation_Case):
     def createSource(self):
         open("testtmp.i", "wt").write("int main() {}")
         
-    def runTest(self):
-        self.runCmd("distcc /bin/true -c testtmp.i", 0)
+    def runtest(self):
+        self.runcmd("distcc true -c testtmp.i", 0)
 
 
 class SBeatsC_Case(CompileHello_Case):
@@ -772,8 +763,8 @@ class SBeatsC_Case(CompileHello_Case):
     If both options are given, we have to make sure we imply the
     output filename in the same way as gcc."""
     # XXX: Are other compilers the same?
-    def runTest(self):
-        self.runCmd("distcc gcc -c -S testtmp.c")
+    def runtest(self):
+        self.runcmd("distcc gcc -c -S testtmp.c")
         if os.path.exists("testtmp.o"):
             self.fail("created testtmp.o but should not have")
         if not os.path.exists("testtmp.s"):
@@ -782,23 +773,19 @@ class SBeatsC_Case(CompileHello_Case):
     
 class NoServer_Case(CompileHello_Case):
     """Invalid server name"""
-    def setUp(self):
-        self.enterRundir()
+    def setup(self):
         self.stripEnvironment()
         os.environ['DISTCC_HOSTS'] = 'no.such.host.here'
         self.distcc_log = 'distcc.log'
         os.environ['DISTCC_LOG'] = self.distcc_log
         self.createSource()
     
-    def runTest(self):
-        self.runCmd("distcc gcc -c -o testtmp.o testtmp.c")
+    def runtest(self):
+        self.runcmd("distcc gcc -c -o testtmp.o testtmp.c")
         msgs = open(self.distcc_log, 'r').read()
-        self.assertReMatch(r'failed to distribute.*running locally instead',
-                           msgs)            
+        self.assert_re_search(r'failed to distribute.*running locally instead',
+                              msgs)            
         
-    def tearDown(self):
-        self.leaveRundir()
-
         
 class ImpliedOutput_Case(CompileHello_Case):
     """Test handling absence of -o"""
@@ -814,18 +801,15 @@ class SyntaxError_Case(Compilation_Case):
 """
 
     def compile(self):
-        rc, msgs = self.runCmdUnchecked(self.compileCmd())
-        self.assertNotEqual(rc, 0)
+        rc, msgs, errs = self.runcmd_unchecked(self.compileCmd())
+        self.assert_notequal(rc, 0)
         # XXX: Need to also handle "syntax error" from gcc-2.95.3
-        if not re.match(r'testtmp.c:1: .*error', msgs):
-            self.fail("compiler error not as expected: \"%s\"" %
-                      msgs)
+        self.assert_re_match(r'testtmp.c:1: .*error', errs)
+        self.assert_equal(msgs, '')
 
-    def linkCmd(self):
-        return ":"                      # don't link
+    def runtest(self):
+        self.compile()
 
-    def checkBuiltProgram(self):
-        """Shouldn't have built"""
         if os.path.exists("testtmp") or os.path.exists("testtmp.o"):
             self.fail("compiler produced output, but should not have done so")
 
@@ -835,16 +819,26 @@ class NoHosts_Case(CompileHello_Case):
 
     We expect compilation to succeed, but with a warning that it was
     run locally."""
-    def setupEnv(self):
-        # WithDaemon_Case sets this to point to the local host, but we don't.
-        pass
+    def runtest(self):
+        import os
+        
+        # WithDaemon_Case sets this to point to the local host, but we
+        # don't want that.  Note that you cannot delete environment
+        # keys in Python1.5, so we need to just set them to the empty
+        # string.
+        os.environ['DISTCC_HOSTS'] = ''
+        os.environ['DISTCC_LOG'] = ''
+        self.runcmd('printenv')
+        msgs, errs = self.runcmd(self.compileCmd())
+
+        # We expect only one message, a warning from distcc
+        self.assert_re_search(r"Warning.*\$DISTCC_HOSTS.*can't distribute work",
+                              errs)
+
+    def compileCmd(self):
+        """Return command to compile source and run tests"""
+        return "DISTCC_FALLBACK=1 distcc cc -o testtmp.o -c %s" % (self.sourceFilename())
 
-    def checkCompileMsgs(self, msgs):
-        "We expect only one message, a warning from distcc"
-        if not re.search(r"Warning.*\$DISTCC_HOSTS.*can't distribute work",
-                         msgs):
-            self.fail("did not find expected warning about HOSTS in \"%s\"; %s"
-                      % (msgs, pprint.pformat(os.environ)))
 
 
 class MissingCompiler_Case(CompileHello_Case):
@@ -858,9 +852,10 @@ class MissingCompiler_Case(CompileHello_Case):
     def source(self):
         return """int foo;"""
 
-    def runTest(self):
-        msgs = self.runCmd("distcc nosuchcc -c testtmp.i", expectedResult=EXIT_COMPILER_MISSING)
-        self.assert_regexp(r'failed to exec.*No such file or directory', msgs)
+    def runtest(self):
+        msgs, errs = self.runcmd("DISTCC_FALLBACK=0 distcc nosuchcc -c testtmp.i",
+                                 expectedResult=EXIT_COMPILER_MISSING)
+        self.assert_re_search(r'failed to exec.*No such file or directory', errs)
         
 
 
@@ -900,18 +895,17 @@ int main(void) {
 """
     asm_filename = 'test2.s'
 
-    def setUp(self):
-        CompileHello_Case.setUp(self)
+    def setup(self):
+        CompileHello_Case.setup(self)
         open(self.asm_filename, 'wt').write(self.asm_source)
         
         # TODO: Delete temp files
 
     def compile(self):
         # Need to build both the C file and the assembly file
-        msgs = (self.runCmd("distcc cc -o test2.o -c test2.s")
-                + self.runCmd("distcc cc -o testtmp.o -c testtmp.c")
-                + self.runCmd(self.linkCmd()))
-        self.checkCompileMsgs(msgs)
+        self.runcmd("distcc cc -o test2.o -c test2.s")
+        self.runcmd("distcc cc -o testtmp.o -c testtmp.c")
+        self.runcmd(self.linkCmd())
 
     def asmCmd(self):
         return "distcc cc -o test2.o -c %s" % (self.asm_filename)
@@ -939,22 +933,21 @@ msg:
     asm_filename = 'test2.S'
     
     def compile(self):
-        msgs = (self.runCmd("distcc cc -o test2.o -c test2.S")
-                + self.runCmd("distcc cc -o testtmp.o -c testtmp.c")
-                + self.runCmd("distcc cc -o testtmp testtmp.o test2.o"))
-        self.checkCompileMsgs(msgs)
+        self.runcmd("distcc cc -o test2.o -c test2.S")
+        self.runcmd("distcc cc -o testtmp.o -c testtmp.c")
+        self.runcmd("distcc cc -o testtmp testtmp.o test2.o")
 
 
 class ModeBits_Case(CompileHello_Case):
     """Check distcc obeys umask"""
-    def runTest(self):
-        self.runCmd("umask 0; distcc -c testtmp.c")
-        self.assertEquals(S_IMODE(os.stat("testtmp.o")[ST_MODE]), 0666)
+    def runtest(self):
+        self.runcmd("umask 0; distcc -c testtmp.c")
+        self.assert_equal(S_IMODE(os.stat("testtmp.o")[ST_MODE]), 0666)
 
 
 class CheckRoot_Case(comfychair.TestCase):
     """Stub case that checks this is run by root.  Not used by default."""
-    def setUp(self):
+    def setup(self):
         self.require_root()
 
 
@@ -967,60 +960,127 @@ class EmptySource_Case(Compilation_Case):
     def source(self):
         return ''
 
-    def runTest(self):
+    def runtest(self):
         self.compile()
 
     def compile(self):
-        self.runCmd("distcc cc -c %s" % self.sourceFilename())
+        self.runcmd("distcc cc -c %s" % self.sourceFilename())
 
     def sourceFilename(self):
         return "testtmp.i"
 
+class BadLogFile_Case(comfychair.TestCase):
+    def runtest(self):
+        self.runcmd("touch distcc.log")
+        self.runcmd("chmod 0 distcc.log")
+        msgs, errs = self.runcmd("DISTCC_LOG=distcc.log distcc -c foo.c",
+                                expectedResult=1)
+        self.assert_re_search("failed to open logfile", errs)
 
 
+class AccessDenied_Case(CompileHello_Case):
+    """Run the daemon, but don't allow access from this host.
+
+    Make sure that compilation falls back to localhost with a warning."""
+    def daemon_command(self):
+        return ("distccd --verbose --lifetime=%d --daemon --log-file %s "
+                "--pid-file %s --port %d --allow 127.0.0.2"
+                % (self.daemon_lifetime(),
+                   self.daemon_logfile, self.daemon_pidfile, self.server_port))
+
+    def compileCmd(self):
+        """Return command to compile source and run tests"""
+        return "DISTCC_FALLBACK=1 distcc cc -o testtmp.o -c %s" % (self.sourceFilename())
+
+    
+    def runtest(self):
+        self.compile()
+        errs = open('distcc.log').read()
+        self.assert_re_search(r'failed to distribute', errs)
+
+
+class ParseMask_Case(comfychair.TestCase):
+    """Test code for matching IP masks."""
+    values = [
+        ('127.0.0.1', '127.0.0.1', 0),
+        ('127.0.0.1', '127.0.0.0', EXIT_ACCESS_DENIED),
+        ('127.0.0.1', '127.0.0.2', EXIT_ACCESS_DENIED),
+        ('127.0.0.1/8', '127.0.0.2', 0),
+        ('10.113.0.0/16', '10.113.45.67', 0),
+        ('10.113.0.0/16', '10.11.45.67', EXIT_ACCESS_DENIED),
+        ('10.113.0.0/16', '127.0.0.1', EXIT_ACCESS_DENIED),
+        ('1.2.3.4/0', '4.3.2.1', EXIT_BAD_ARGUMENTS),
+        ('1.2.3.4/40', '4.3.2.1', EXIT_BAD_ARGUMENTS),
+        ('1.2.3.4.5.6.7/8', '127.0.0.1', EXIT_BAD_ARGUMENTS),
+        ('1.2.3.4/8', '4.3.2.1', EXIT_ACCESS_DENIED),
+        ('192.168.1.64/28', '192.168.1.70', 0),
+        ('192.168.1.64/28', '192.168.1.7', EXIT_ACCESS_DENIED),
+        ]
+    def runtest(self):
+        for mask, client, expected in ParseMask_Case.values:
+            ret, msgs, err = self.runcmd_unchecked("h_parsemask %s %s" % (mask, client))
+            if ret != expected:
+                self.fail("%s %s -> %d" % (mask, client, ret))
+
+
+class HostFile_Case(CompileHello_Case):
+    def setup(self):
+        CompileHello_Case.setup(self)
+        del os.environ['DISTCC_HOSTS']
+        self.save_home = os.environ['HOME']
+        os.environ['HOME'] = os.getcwd()
+        os.mkdir('.distcc')
+        open('.distcc/hosts', 'w').write('127.0.0.1:%d' % self.server_port)
+
+    def teardown(self):
+        os.environ['HOME'] = self.save_home
+        CompileHello_Case.teardown(self)
+
+
+# All the tests defined in this suite
+tests = [BadLogFile_Case,
+         ScanArgs_Case,
+         ParseMask_Case,
+         ImplicitCompilerScan_Case,
+         StripArgs_Case,
+         StartStopDaemon_Case,
+         CompileHello_Case,
+         CppError_Case,
+         BadInclude_Case,
+         PreprocessPlainText_Case,
+         NoDetachDaemon_Case,
+         SBeatsC_Case,
+         DashD_Case,
+         BinFalse_Case,
+         BinTrue_Case,
+         VersionOption_Case,
+         HelpOption_Case,
+         BogusOption_Case,
+         MultipleCompile_Case,
+         GccOptionsPassed_Case,
+         IsSource_Case,
+         ExtractExtension_Case,
+         ImplicitCompiler_Case,
+         DaemonBadPort_Case,
+         AccessDenied_Case,
+         NoServer_Case,
+         InvalidHostSpec_Case,
+         ParseHostSpec_Case,
+         ImpliedOutput_Case,
+         SyntaxError_Case,
+         NoHosts_Case,
+         MissingCompiler_Case,
+         RemoteAssemble_Case,
+         PreprocessAsm_Case,
+         ModeBits_Case,
+         EmptySource_Case,
+         HostFile_Case,
+
+         # slow tests below here
+         Concurrent_Case,
+         ThousandFold_Case,
+         BigAssFile_Case]
 
 
 if __name__ == '__main__':
-    import sys
-    
-    if len(sys.argv) > 1:
-        fns = map(locals().get, sys.argv[1:])
-        ret = comfychair.runtests(fns)
-    else:
-        ret = comfychair.runtests([ScanArgs_Case,
-                                   ImplicitCompilerScan_Case,
-                                   StripArgs_Case,
-                                   StartStopDaemon_Case,
-                                   CompileHello_Case,
-                                   CppError_Case,
-                                   BadInclude_Case,
-                                   PreprocessPlainText_Case,
-                                   NoDetachDaemon_Case,
-                                   SBeatsC_Case,
-                                   DashD_Case,
-                                   BinFalse_Case,
-                                   BinTrue_Case,
-                                   VersionOption_Case,
-                                   HelpOption_Case,
-                                   BogusOption_Case,
-                                   MultipleCompile_Case,
-                                   GccOptionsPassed_Case,
-                                   IsSource_Case,
-                                   ExtractExtension_Case,
-                                   ImplicitCompiler_Case,
-                                   DaemonBadPort_Case,
-                                   NoServer_Case,
-                                   InvalidHostSpec_Case,
-                                   ParseHostSpec_Case,
-                                   ImpliedOutput_Case,
-                                   SyntaxError_Case,
-                                   NoHosts_Case,
-                                   MissingCompiler_Case,
-                                   RemoteAssemble_Case,
-                                   PreprocessAsm_Case,
-                                   ModeBits_Case,
-                                   EmptySource_Case,
-                                   Concurrent_Case,
-                                   ThousandFold_Case,
-                                   BigAssFile_Case])
-    sys.exit(ret)
+    comfychair.main(tests)
